from django.contrib import admin
from django.db.models import Q, QuerySet
from django.template.loader import render_to_string
from django.urls import reverse
from edc_model_admin.dashboard import ModelAdminDashboardMixin
from edc_model_admin.mixins import TemplatesModelAdminMixin
from edc_qareports.modeladmin_mixins import QaReportModelAdminMixin
from edc_sites.admin import SiteModelAdminMixin

from ..vl import VlSummary
from .list_filters import (
    BaselineVlDateListFilter,
    BaselineVlListFilter,
    EndlineVlDateListFilter,
    EndlineVlListFilter,
    LastVlDateListFilter,
    NextVlDateListFilter,
    ScheduleStatusListFilter,
)


class VlSummaryModelAdminMixin(
    QaReportModelAdminMixin,
    SiteModelAdminMixin,
    ModelAdminDashboardMixin,
    TemplatesModelAdminMixin,
):
    report_model: str = None
    endline_months: int = None

    qa_report_list_display_insert_pos = 2
    ordering = ["site", "subject_identifier"]
    list_display = [
        "dashboard",
        "subject",
        "baseline_vl",
        "endline_vl",
        "expected",
        "baseline_date",
        "baseline_vl_date",
        "endline_vl_date",
        "last_vl_date",
        "next_vl_date",
        "offschedule_date",
        "offset",
        "created",
    ]

    list_filter = [
        "expected",
        BaselineVlListFilter,
        EndlineVlListFilter,
        ScheduleStatusListFilter,
        "baseline_date",
        BaselineVlDateListFilter,
        EndlineVlDateListFilter,
        LastVlDateListFilter,
        NextVlDateListFilter,
        "offset",
    ]

    search_fields = ["subject_identifier"]

    @admin.display(description="Subject", ordering="subject_identifier")
    def subject(self, obj):
        url = reverse("intecomm_reports_admin:intecomm_reports_vl_changelist")
        context = dict(url=url, subject_identifier=obj.subject_identifier)
        return render_to_string(
            "intecomm_reports/columns/vl_subject_identifier.html", context=context
        )

    def get_queryset(self, request) -> QuerySet:
        vl_summary = VlSummary(endline_months=self.endline_months, skip_update_dx=True)
        vl_summary.to_model(model=self.report_model)
        qs = super().get_queryset(request)
        return qs.filter((Q(baseline_vl__isnull=True) | Q(endline_vl__isnull=True)))



from django.contrib import admin
from edc_model_admin.dashboard import ModelAdminDashboardMixin
from edc_model_admin.mixins import TemplatesModelAdminMixin
from edc_qareports.modeladmin_mixins import QaReportModelAdminMixin
from edc_sites.admin import SiteModelAdminMixin
from edc_visit_schedule.admin import ScheduleStatusListFilter

from ...admin_site import intecomm_reports_admin
from ...models import SubjectsTransferred


@admin.register(SubjectsTransferred, site=intecomm_reports_admin)
class SubjectsTransferredAdmin(
    QaReportModelAdminMixin,
    SiteModelAdminMixin,
    ModelAdminDashboardMixin,
    TemplatesModelAdminMixin,
    admin.ModelAdmin,
):
    qa_report_list_display_insert_pos = 2
    ordering = ["site", "subject_identifier"]
    list_display = [
        "dashboard",
        "subject_identifier",
        "last_visit",
        "code",
        "consented",
        "transferred",
        "m",
        "last_seen",
        "offstudy",
    ]

    list_filter = [
        ScheduleStatusListFilter,
        "visit_code",
        "transferred",
    ]

    search_fields = ["subject_identifier"]

    @admin.display(description="M", ordering="months")
    def m(self, obj=None):
        return obj.months

    @admin.display(description="Code", ordering="visit_code")
    def code(self, obj=None):
        return obj.visit_code



# Generated by Django 4.2.11 on 2024-06-20 15:59

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="SubjectsTransferred",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("report_model", models.CharField(max_length=50)),
                ("subject_identifier", models.CharField(max_length=25)),
                ("created", models.DateTimeField()),
                ("consented", models.DateField()),
                ("visit", models.DateField()),
                ("visit_code", models.CharField(max_length=100)),
                ("transferred", models.DateField(blank=True, null=True)),
                ("months", models.IntegerField(blank=True, null=True)),
                ("offstudy", models.DateField(blank=True, null=True)),
                ("offstudy_report", models.DateField(blank=True, null=True)),
            ],
            options={
                "verbose_name": "Subjects Transferred",
                "verbose_name_plural": "Subjects Transferred",
                "db_table": "subjects_transferred_view",
                "managed": False,
            },
        ),
    ]



# Generated by Django 5.0 on 2024-01-25 05:33

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0067_auto_20231216_1807"),
        ("sites", "0002_alter_domain_unique"),
    ]

    operations = [
        migrations.AlterField(
            model_name="consentrefusal",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="healthtalklog",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalconsentrefusal",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealthtalklog",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicaloldhealthfacility",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalpatientcall",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="oldhealthfacility",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="patientcall",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="patientlogreportprinthistory",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
    ]



from django.db import models
from edc_crf.model_mixins import CrfStatusModelMixin
from edc_lab.model_mixins import CrfWithRequisitionModelMixin, requisition_fk_options
from edc_lab_panel.panels import lft_panel
from edc_lab_results import BLOOD_RESULTS_LFT_ACTION
from edc_lab_results.model_mixins import (
    AlbuminModelMixin,
    AlpModelMixin,
    AltModelMixin,
    AmylaseModelMixin,
    AstModelMixin,
    BloodResultsModelMixin,
    GgtModelMixin,
)
from edc_model.models import BaseUuidModel

from ...model_mixins import CrfWithActionModelMixin


class BloodResultsLft(
    CrfWithActionModelMixin,
    AlbuminModelMixin,
    AlpModelMixin,
    AltModelMixin,
    AmylaseModelMixin,
    AstModelMixin,
    GgtModelMixin,
    CrfWithRequisitionModelMixin,
    BloodResultsModelMixin,
    CrfStatusModelMixin,
    BaseUuidModel,
):
    action_name = BLOOD_RESULTS_LFT_ACTION
    tracking_identifier_prefix = "LF"
    lab_panel = lft_panel

    requisition = models.ForeignKey(
        limit_choices_to={"panel__name": lft_panel.name}, **requisition_fk_options
    )

    class Meta(CrfWithActionModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Blood Result: LFT"
        verbose_name_plural = "Blood Results: LFT"
        indexes = CrfWithActionModelMixin.Meta.indexes + BaseUuidModel.Meta.indexes



from edc_qareports.sql_generator import SqlViewGenerator


def get_view_definition() -> dict:
    sql_view = SqlViewGenerator(
        report_model="intecomm_reports.eos_view",
        ordering=["site_id", "subject_identifier"],
    )

    subquery = """
    select subject_identifier, visit_datetime, timepoint, visit_code, visit_code_sequence,
    offstudy_reason, offstudy_datetime, site_id, TIMESTAMPDIFF(MONTH, visit_datetime,
    now()) as m, schedule_status
    from (
        select b1.subject_identifier, max(b1.visit_datetime) as visit_datetime,
        max(b1.timepoint) as timepoint, max(eos.offstudy_datetime) as offstudy_datetime,
        min(baseline_datetime) as baseline_datetime, sched.schedule_status,
        reasons.name as offstudy_reason, appt.visit_code, appt.visit_code_sequence, appt.site_id
		from (
			select v.subject_identifier, max(v.report_datetime) as visit_datetime,
			min(v.report_datetime) as baseline_datetime, max(appt.timepoint) as timepoint
			from intecomm_screening_patientlog as plog
			left join intecomm_subject_subjectvisit as v on v.subject_identifier=plog.subject_identifier
			left join edc_appointment_appointment as appt on v.appointment_id=appt.id
			where plog.group_identifier is not null
			group by v.subject_identifier
        ) as b1
		left join edc_appointment_appointment as appt on (b1.subject_identifier=appt.subject_identifier and b1.timepoint=appt.timepoint)
        left join intecomm_prn_endofstudy as eos on eos.subject_identifier=b1.subject_identifier
        left join intecomm_lists_offstudyreasons as reasons on eos.id=reasons.id
        left join edc_visit_schedule_subjectschedulehistory as sched on b1.subject_identifier=sched.subject_identifier
        group by b1.subject_identifier, sched.schedule_status, reasons.name, appt.visit_code, appt.visit_code_sequence, appt.site_id
        order by b1.subject_identifier
    ) as b2
        """  # noqa

    return {
        "django.db.backends.mysql": sql_view.as_mysql(subquery),
        "django.db.backends.postgresql": sql_view.as_postgres(subquery),
        "django.db.backends.sqlite3": sql_view.as_sqlite(subquery),
    }



# Generated by Django 4.2.7 on 2023-12-05 20:38

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_prn", "0012_alter_endofstudy_options_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="endofstudy",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="endofstudy",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalendofstudy",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalendofstudy",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicallosstofollowup",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicallosstofollowup",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicaloffschedulecomm",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicaloffschedulecomm",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicaloffscheduleinte",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicaloffscheduleinte",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalonschedulecomm",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalonschedulecomm",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalonscheduleinte",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalonscheduleinte",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalprotocolincident",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalprotocolincident",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectlocator",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjectlocator",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjecttransfer",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalsubjecttransfer",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="losstofollowup",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="losstofollowup",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="offschedulecomm",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="offschedulecomm",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="offscheduleinte",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="offscheduleinte",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="onschedulecomm",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="onschedulecomm",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="onscheduleinte",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="onscheduleinte",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="protocolincident",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="protocolincident",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="subjecttransfer",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="subjecttransfer",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
    ]



# Generated by Django 4.2.5 on 2023-10-05 23:38

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0120_remove_historicalnextappointment_info_source_new_and_more"),
    ]

    operations = [
        migrations.RenameField(
            model_name="historicalnextappointment",
            old_name="info_source_new",
            new_name="info_source",
        ),
        migrations.RenameField(
            model_name="nextappointment",
            old_name="info_source_new",
            new_name="info_source",
        ),
    ]



# Generated by Django 4.2.5 on 2023-10-05 23:44

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0121_auto_20231006_0238"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="historicalnextappointment",
            name="info_source_old",
        ),
        migrations.RemoveField(
            model_name="nextappointment",
            name="info_source_old",
        ),
    ]



from edc_glucose.model_mixins import (
    fasting_model_mixin_factory,
    glucose_model_mixin_factory,
)
from edc_model.models import BaseUuidModel

from ..model_mixins import CrfModelMixin, FollowupReviewModelMixin


class DmReview(
    FollowupReviewModelMixin,
    glucose_model_mixin_factory("glucose"),
    fasting_model_mixin_factory(
        "glucose",
        verbose_names=dict(
            glucose_fasting="Was glucose measured while fasting?",
            glucose_fasting_duration_str="How long did they fast (in hours and minutes)?",
        ),
    ),
    CrfModelMixin,
    BaseUuidModel,
):
    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Diabetes Review"
        verbose_name_plural = "Diabetes Review"
        indexes = CrfModelMixin.Meta.indexes + BaseUuidModel.Meta.indexes



from django.contrib.auth.models import Permission, User
from django.contrib.sites.models import Site
from django.test import override_settings
from django.urls import reverse
from django_webtest import WebTest
from edc_dashboard.url_names import url_names

from intecomm_dashboard.views import SubjectDashboardView
from intecomm_screening.tests.intecomm_test_case_mixin import IntecommTestCaseMixin
from intecomm_subject.models import DrugSupplyDm, DrugSupplyHiv, DrugSupplyHtn
from intecomm_subject.models import HealthEconomics as OldHealthEconomics


class TestSubjectDashboard(IntecommTestCaseMixin, WebTest):
    def setUp(self) -> None:
        super().setUp()
        self.user = User.objects.create_superuser("user_login", "u@example.com", "pass")
        self.user.is_active = True
        self.user.is_staff = True
        self.user.is_superuser = True
        self.user.save()
        self.user.refresh_from_db()
        self.exclude_models = [
            DrugSupplyDm,
            DrugSupplyHtn,
            DrugSupplyHiv,
            OldHealthEconomics,
        ]
        self.user.userprofile.sites.add(Site.objects.get(id=101))
        self.user.userprofile.sites.add(Site.objects.get(id=201))
        self.user.user_permissions.add(Permission.objects.get(codename="view_appointment"))

    def login(self):
        form = None
        response = self.app.get(reverse("admin:index")).maybe_follow()
        for index, form in response.forms.items():
            if form.action == "/i18n/setlang/":
                # exclude the locale form
                continue
            else:
                break
        form["username"] = self.user.username
        form["password"] = "pass"  # nosec B105
        return form.submit()

    @override_settings(SITE_ID=201)
    def test_dashboard_ok(self):
        subject_screening = self.get_subject_screening()
        self.assertEqual(subject_screening.reasons_ineligible, None)
        self.assertTrue(subject_screening.eligible)

        subject_consent = self.get_subject_consent(subject_screening)
        self.assertIsNotNone(subject_consent.subject_identifier)

        self.login()

        url_name = url_names.get(SubjectDashboardView.dashboard_url_name)
        url = reverse(
            url_name, kwargs=dict(subject_identifier=subject_consent.subject_identifier)
        )
        self.app.get(url, user=self.user, status=200)

    @override_settings(SITE_ID=101)
    def test_dashboard_ug_ok(self):
        subject_screening = self.get_subject_screening()
        self.assertEqual(subject_screening.reasons_ineligible, None)
        self.assertTrue(subject_screening.eligible)

        subject_consent = self.get_subject_consent(subject_screening)
        self.assertIsNotNone(subject_consent.subject_identifier)

        self.login()

        url_name = url_names.get(SubjectDashboardView.dashboard_url_name)
        url = reverse(
            url_name, kwargs=dict(subject_identifier=subject_consent.subject_identifier)
        )
        self.app.get(url, user=self.user, status=200)



from django.apps import AppConfig as DjangoApponfig


class AppConfig(DjangoApponfig):
    name = "intecomm_auth"
    verbose_name = "INTECOMM: Authentication and Permissions"
    default_auto_field = "django.db.models.BigAutoField"



from edc_qareports.sql_generator import SqlViewGenerator


def get_view_definition() -> dict:
    with_stmt = """
        with visits as (
                select subject_identifier, report_datetime, visit_code, visit_code_sequence
                from intecomm_subject_subjectvisit
                group by subject_identifier, report_datetime, visit_code, visit_code_sequence
                order by subject_identifier, report_datetime, visit_code, visit_code_sequence
            )
    """
    sql_view = SqlViewGenerator(
        report_model="intecomm_reports.subjectstransferred_view",
        ordering=["site_id", "subject_identifier"],
        with_stmt=with_stmt,
    )

    subquery = """
        select prn.subject_identifier,
               prn.site_id,
               consent_datetime                                     as consented,
               v.visit_code,
               v.report_datetime                                    as last_visit,
               prn.report_datetime                                  as transferred,
               datediff(prn.report_datetime, consent_datetime) / 30 as months,
               eof.offstudy_datetime                                as offstudy,
               eof.report_datetime                                  as last_seen
        from intecomm_prn_subjecttransfer as prn
            left join edc_registration_registeredsubject as rs on prn.subject_identifier = rs.subject_identifier
            left join intecomm_prn_endofstudy as eof on prn.subject_identifier = eof.subject_identifier
            left join (
                select distinct subject_identifier,
                last_value(report_datetime) OVER w as report_datetime,
                last_value(visit_code) OVER w as visit_code
                from visits WINDOW w as (PARTITION BY `subject_identifier`)
            ) v on prn.subject_identifier = v.subject_identifier
        order by datediff(prn.report_datetime, consent_datetime) / 30 desc, subject_identifier
        """  # noqa

    return {
        "django.db.backends.mysql": sql_view.as_mysql(subquery),
        "django.db.backends.postgresql": sql_view.as_postgres(subquery),
        "django.db.backends.sqlite3": sql_view.as_sqlite(subquery),
    }



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from edc_model.models import BaseUuidModel, HistoricalRecords
from edc_model_fields.fields import OtherCharField
from edc_sites.managers import CurrentSiteManager
from edc_sites.model_mixins import SiteModelMixin
from edc_utils import get_utcnow

from intecomm_lists.models import HealthFacilityTypes


class Manager(models.Manager):
    use_in_migrations = True

    def get_by_natural_key(self, name):
        return self.get(name=name)


# TODO: This model is not used. See proxy in intecomm_facility
#   Remove after migrations are squashed/reset
class OldHealthFacility(SiteModelMixin, BaseUuidModel):
    report_datetime = models.DateTimeField(default=get_utcnow)

    name = models.CharField(max_length=25, unique=True)

    health_facility_type = models.ForeignKey(
        HealthFacilityTypes,
        verbose_name="Health facility type",
        on_delete=models.PROTECT,
    )

    health_facility_type_other = OtherCharField()

    distance = models.IntegerField(
        verbose_name="Approximate distance from integrated-care clinic",
        help_text="In km. If within the integrated-care clinic facility type '0'",
    )

    gps = models.CharField(
        max_length=50,
        null=True,
        blank=True,
        help_text="copy and paste directly from google maps",
    )

    latitude = models.FloatField(
        validators=[MinValueValidator(-90), MaxValueValidator(90)],
        null=True,
        blank=True,
        help_text="in degrees. copy and paste directly from google maps",
    )

    longitude = models.FloatField(
        validators=[MinValueValidator(-180), MaxValueValidator(180)],
        null=True,
        blank=True,
        help_text="in degrees. copy and paste directly from google maps",
    )

    mon = models.BooleanField()
    tue = models.BooleanField()
    wed = models.BooleanField()
    thu = models.BooleanField()
    fri = models.BooleanField()
    sat = models.BooleanField()

    notes = models.TextField(null=True, blank=True)

    on_site = CurrentSiteManager()
    history = HistoricalRecords()
    objects = Manager()

    def __str__(self):
        return f"{self.name} {self.health_facility_type.display_name}"

    def save(self, *args, **kwargs):
        self.name = self.name.upper()
        super().save(*args, **kwargs)

    def natural_key(self):
        return (self.name,)

    @property
    def clinic_days(self) -> list[int]:
        """Using ISO numbering where Monday=1."""
        days = []
        if self.mon:
            days.append(1)
        if self.tue:
            days.append(2)
        if self.wed:
            days.append(3)
        if self.thu:
            days.append(4)
        if self.fri:
            days.append(5)
        if self.sat:
            days.append(6)
        return days

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Health Facility"
        verbose_name_plural = "Health Facilities"



# Generated by Django 4.2.11 on 2024-05-22 21:30

from django.db import migrations, models
import django.db.models.manager
import edc_sites.managers


class Migration(migrations.Migration):

    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        ("intecomm_subject", "0183_alter_dmcurrentconditions_dm_and_more"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="DmCurrentConditions",
            new_name="CurrentConditions",
        ),
        migrations.AlterModelManagers(
            name="currentconditions",
            managers=[
                ("objects", django.db.models.manager.Manager()),
                ("on_site", edc_sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.RenameIndex(
            model_name="currentconditions",
            new_name="intecomm_su_modifie_490726_idx",
            old_name="intecomm_su_modifie_12b4ab_idx",
        ),
        migrations.RenameIndex(
            model_name="currentconditions",
            new_name="intecomm_su_user_mo_15c1bf_idx",
            old_name="intecomm_su_user_mo_596162_idx",
        ),
        migrations.RemoveField(
            model_name="bloodresultsins",
            name="fasting_duration_minutes",
        ),
        migrations.RemoveField(
            model_name="dminitialreview",
            name="glucose_fasting_duration_minutes",
        ),
        migrations.RemoveField(
            model_name="dmreview",
            name="glucose_fasting_duration_minutes",
        ),
        migrations.RemoveField(
            model_name="glucose",
            name="glucose_fasting_duration_minutes",
        ),
        migrations.RemoveField(
            model_name="historicalbloodresultsins",
            name="fasting_duration_minutes",
        ),
        migrations.RemoveField(
            model_name="historicaldminitialreview",
            name="glucose_fasting_duration_minutes",
        ),
        migrations.RemoveField(
            model_name="historicaldmreview",
            name="glucose_fasting_duration_minutes",
        ),
        migrations.RemoveField(
            model_name="historicalglucose",
            name="glucose_fasting_duration_minutes",
        ),
        migrations.AddField(
            model_name="bloodresultsins",
            name="fasting_duration_delta",
            field=models.DurationField(
                blank=True,
                help_text="system calculated to microseconds. (hours=microseconds/3.6e+9)",
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="dminitialreview",
            name="glucose_fasting_duration_delta",
            field=models.DurationField(
                blank=True,
                help_text="system calculated to microseconds. (hours=microseconds/3.6e+9)",
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="dmreview",
            name="glucose_fasting_duration_delta",
            field=models.DurationField(
                blank=True,
                help_text="system calculated to microseconds. (hours=microseconds/3.6e+9)",
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="glucose",
            name="glucose_fasting_duration_delta",
            field=models.DurationField(
                blank=True,
                help_text="system calculated to microseconds. (hours=microseconds/3.6e+9)",
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsins",
            name="fasting_duration_delta",
            field=models.DurationField(
                blank=True,
                help_text="system calculated to microseconds. (hours=microseconds/3.6e+9)",
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="historicaldminitialreview",
            name="glucose_fasting_duration_delta",
            field=models.DurationField(
                blank=True,
                help_text="system calculated to microseconds. (hours=microseconds/3.6e+9)",
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="historicaldmreview",
            name="glucose_fasting_duration_delta",
            field=models.DurationField(
                blank=True,
                help_text="system calculated to microseconds. (hours=microseconds/3.6e+9)",
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="historicalglucose",
            name="glucose_fasting_duration_delta",
            field=models.DurationField(
                blank=True,
                help_text="system calculated to microseconds. (hours=microseconds/3.6e+9)",
                null=True,
            ),
        ),
    ]



# Generated by Django 5.0.8 on 2024-08-26 15:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_reports", "0013_auto_20240826_1807"),
    ]

    operations = [
        migrations.CreateModel(
            name="Eos",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("report_model", models.CharField(max_length=50)),
                ("subject_identifier", models.CharField(max_length=25)),
                ("created", models.DateTimeField()),
                ("visit_datetime", models.DateTimeField(blank=True, null=True)),
                (
                    "timepoint",
                    models.DecimalField(
                        blank=True, decimal_places=2, max_digits=10, null=True
                    ),
                ),
                ("visit_code", models.CharField(max_length=25, null=True)),
                ("visit_code_sequence", models.IntegerField(null=True)),
                ("offstudy_reason", models.CharField(blank=True, max_length=200, null=True)),
                ("offstudy_datetime", models.DateTimeField(blank=True, null=True)),
            ],
            options={
                "verbose_name": "End of study",
                "verbose_name_plural": "End of study",
                "db_table": "eos_view",
                "managed": False,
            },
        ),
    ]



# Generated by Django 4.2.10 on 2024-02-26 13:21

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_subject", "0147_alter_careseekinga_accompany_num_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="bloodresultslft",
            name="alp_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(9999.99),
                ],
                verbose_name="ALP",
            ),
        ),
        migrations.AlterField(
            model_name="bloodresultslft",
            name="alt_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(9999.99),
                ],
                verbose_name="ALT",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="alp_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(9999.99),
                ],
                verbose_name="ALP",
            ),
        ),
        migrations.AlterField(
            model_name="historicalbloodresultslft",
            name="alt_value",
            field=models.DecimalField(
                blank=True,
                decimal_places=2,
                max_digits=8,
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(1.0),
                    django.core.validators.MaxValueValidator(9999.99),
                ],
                verbose_name="ALT",
            ),
        ),
    ]



# Generated by Django 4.2.11 on 2024-07-29 21:40

from django.db import migrations
import django.db.models.manager
import edc_sites.managers


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_reports", "0009_missingvldrawdates"),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="missingvldrawdates",
            managers=[
                ("objects", django.db.models.manager.Manager()),
                ("on_site", edc_sites.managers.CurrentSiteManager()),
            ],
        ),
    ]



from edc_locator.models import SubjectLocator as BaseModel


class SubjectLocator(BaseModel):
    class Meta:
        proxy = True
        verbose_name = "Subject Locator"
        verbose_name_plural = "Subject Locators"



from django.apps import AppConfig as DjangoAppConfig
from django.contrib.admin.apps import AdminConfig as DjangoAdminConfig
from django.core.management.color import color_style

style = color_style()


class AdminConfig(DjangoAdminConfig):
    default_site = "intecomm_edc.admin.AdminSite"


class AppConfig(DjangoAppConfig):
    name = "intecomm_edc"
    verbose_name = "INTECOMM"



# Generated by Django 4.2.7 on 2023-12-04 22:19

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_ae", "0010_alter_deathreport_death_date_and_more"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="aefollowup",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "AE Followup",
                "verbose_name_plural": "AE Followups",
            },
        ),
        migrations.AlterModelOptions(
            name="aeinitial",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
            },
        ),
        migrations.AlterModelOptions(
            name="aesusar",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "AE SUSAR Report",
                "verbose_name_plural": "AE SUSAR Reports",
            },
        ),
        migrations.AlterModelOptions(
            name="aetmg",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "AE TMG Report",
                "verbose_name_plural": "AE TMG Reports",
            },
        ),
        migrations.AlterModelOptions(
            name="deathreport",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Death Report",
                "verbose_name_plural": "Death Reports",
            },
        ),
        migrations.AlterModelOptions(
            name="deathreporttmg",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Death Report TMG (1st)",
                "verbose_name_plural": "Death Report TMG (1st)",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalaefollowup",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical AE Followup",
                "verbose_name_plural": "historical AE Followups",
            },
        ),
        migrations.AlterModelOptions(
            name="historicalaeinitial",
            options={
                "get_latest_by": ("history_date", "history_id"),
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical ae initial",
                "verbose_name_plural": "historical ae initials",
            },
        ),
        migrations.AlterModelOptions(
            name="hospitalization",
            options={
                "default_manager_name": "objects",
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "verbose_name": "Hospitalization",
                "verbose_name_plural": "Hospitalization",
            },
        ),
        migrations.RemoveIndex(
            model_name="aeinitial",
            name="intecomm_ae_subject_222cd1_idx",
        ),
        migrations.AddField(
            model_name="aefollowup",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="aefollowup",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="aeinitial",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="aeinitial",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="aesusar",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="aesusar",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="aetmg",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="aetmg",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="deathreport",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="deathreport",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="deathreporttmg",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="deathreporttmg",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalaefollowup",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalaefollowup",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalaeinitial",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalaeinitial",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalaesusar",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalaesusar",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalaetmg",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalaetmg",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicaldeathreport",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicaldeathreport",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicaldeathreporttmg",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicaldeathreporttmg",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicaldeathreporttmgsecond",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicaldeathreporttmgsecond",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="historicalhospitalization",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="historicalhospitalization",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
        migrations.AddField(
            model_name="hospitalization",
            name="locale_created",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale created",
            ),
        ),
        migrations.AddField(
            model_name="hospitalization",
            name="locale_modified",
            field=models.CharField(
                blank=True,
                help_text="Auto-updated by Modeladmin",
                max_length=10,
                null=True,
                verbose_name="Locale modified",
            ),
        ),
    ]



from django.core.validators import MaxValueValidator, MinValueValidator
from django.utils.translation import gettext as _
from edc_model_fields.fields import IntegerField2


class IncomeField(IntegerField2):
    def __init__(self, *args, **kwargs):
        kwargs.update(validators=[MinValueValidator(0), MaxValueValidator(9999999)])
        default_help_text = _(
            "In local currency. Ask for cash value or equivalent cash value for in-kind"
        )

        self.custom_help_text = kwargs.get("help_text")
        kwargs.update(help_text=self.custom_help_text or default_help_text)
        super().__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        del kwargs["validators"]
        if not self.custom_help_text:
            del kwargs["help_text"]
        return name, path, args, kwargs



from django.contrib import admin
from edc_model_admin.dashboard import ModelAdminDashboardMixin
from edc_model_admin.mixins import TemplatesModelAdminMixin
from edc_qareports.modeladmin_mixins import QaReportModelAdminMixin
from edc_sites.admin import SiteModelAdminMixin

from ...admin_site import intecomm_reports_admin
from ...models import Eos


@admin.register(Eos, site=intecomm_reports_admin)
class EosAdmin(
    QaReportModelAdminMixin,
    SiteModelAdminMixin,
    ModelAdminDashboardMixin,
    TemplatesModelAdminMixin,
    admin.ModelAdmin,
):
    include_note_column = False
    ordering = ["site", "subject_identifier"]

    list_display = [
        "dashboard",
        "subject",
        "visit",
        "visit_date",
        "offstudy_date",
        "offstudy_reason",
        "m",
        "schedule_status",
        "site",
        "created",
    ]

    list_filter = [
        "visit_datetime",
        "offstudy_datetime",
        "schedule_status",
        "offstudy_reason",
        "visit_code",
        "visit_code_sequence",
        "m",
    ]

    search_fields = ["subject_identifier"]

    @admin.display(description="Subject", ordering="subject_identifier")
    def subject(self, obj):
        return obj.subject_identifier

    @admin.display(description="Last visit", ordering="visit_code")
    def visit(self, obj):
        return f"{obj.visit_code}.{obj.visit_code_sequence}"

    @admin.display(description="Visit date", ordering="visit_datetime")
    def visit_date(self, obj):
        if obj.visit_datetime:
            return obj.visit_datetime.date()

        return None

    @admin.display(description="Offstudy date", ordering="offstudy_datetime")
    def offstudy_date(self, obj):
        if obj.offstudy_datetime:
            return obj.offstudy_datetime.date()
        return None

    @admin.display(description="Months", ordering="~m")
    def months(self, obj):
        return abs(obj.m)



# Generated by Django 5.0 on 2023-12-16 15:07
from django.apps import apps as django_apps
from django.db import migrations
from django.db.models import Count
from tqdm import tqdm


def update_site(apps, schema_editor):
    model_cls = django_apps.get_model("intecomm_screening.patientlogreportprinthistory")
    patient_log_model_cls = django_apps.get_model("intecomm_screening.patientlog")
    patient_log_identifiers = model_cls.objects.values("patient_log_identifier").annotate(
        count=Count("patient_log_identifier")
    )
    total = len(patient_log_identifiers)
    for data in tqdm(patient_log_identifiers, total=total):
        patient_log = patient_log_model_cls.objects.get(
            patient_log_identifier=data.get("patient_log_identifier")
        )
        model_cls.objects.filter(
            patient_log_identifier=data.get("patient_log_identifier")
        ).update(site=patient_log.site)


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0066_alter_patientlogreportprinthistory_managers_and_more"),
    ]

    operations = [migrations.RunPython(update_site)]



# Generated by Django 4.2.1 on 2023-06-30 03:10

from django.db import migrations, models
import edc_utils


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_screening", "0035_auto_20230630_0552"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalpatientlog",
            name="screening_identifier",
            field=models.CharField(
                db_index=True,
                default=edc_utils.get_uuid,
                help_text="Auto populated when screening form is complete",
                max_length=36,
            ),
        ),
        migrations.AlterField(
            model_name="historicalpatientlog",
            name="subject_identifier",
            field=models.CharField(
                db_index=True,
                default=edc_utils.get_uuid,
                help_text="Auto populated when consent form is complete",
                max_length=36,
            ),
        ),
        migrations.AlterField(
            model_name="patientlog",
            name="screening_identifier",
            field=models.CharField(
                default=edc_utils.get_uuid,
                help_text="Auto populated when screening form is complete",
                max_length=36,
                unique=True,
            ),
        ),
        migrations.AlterField(
            model_name="patientlog",
            name="subject_identifier",
            field=models.CharField(
                default=edc_utils.get_uuid,
                help_text="Auto populated when consent form is complete",
                max_length=36,
                unique=True,
            ),
        ),
    ]



# Generated by Django 4.2.3 on 2023-07-26 13:31

from django.db import migrations
from tqdm import tqdm


def update_health_talk(apps, schema_editor):
    """Update healthfacility FK from old healthfacility model to
    new healthfacility model.
    """
    model_cls = apps.get_model("intecomm_screening", "healthtalklog")
    healthfacility_new_model_cls = apps.get_model("intecomm_facility", "healthfacility")
    healthfacility_old_model_cls = apps.get_model("intecomm_screening", "oldhealthfacility")
    total = model_cls.objects.all().count()
    print("\n")
    for obj in tqdm(model_cls.objects.all(), total=total):
        if not obj.old_health_facility:
            continue
        old_health_facility = healthfacility_old_model_cls.objects.get(
            id=obj.old_health_facility
        )
        obj.health_facility_id = healthfacility_new_model_cls.objects.get(
            name=old_health_facility.name
        ).id
        obj.save_base(update_fields=["health_facility_id"])
    print("\n")


class Migration(migrations.Migration):
    dependencies = [
        # ("intecomm_facility", "0002_alter_healthfacility_device_created_and_more"),
        (
            "intecomm_screening",
            "0050_delete_healthfacility_healthtalklog_health_facility_and_more",
        ),
        ("intecomm_subject", "0067_auto_20230726_0008"),
    ]

    operations = [migrations.RunPython(update_health_talk)]



from django import forms
from django.utils.translation import gettext_lazy as _
from edc_appointment.constants import SKIPPED_APPT
from edc_appointment.forms import AppointmentForm as BaseForm
from edc_appointment.models import AppointmentType
from edc_constants.constants import CLINIC, COMMUNITY
from edc_visit_schedule.utils import is_baseline
from intecomm_rando.constants import COMMUNITY_ARM, FACILITY_ARM
from intecomm_rando.utils import get_assignment_for_subject


class AppointmentForm(BaseForm):
    @property
    def assignment_for_subject(self):
        return get_assignment_for_subject(self.instance.subject_identifier)

    def clean(self):
        cleaned_data = super().clean()
        appt_status = self.cleaned_data.get("appt_status")
        appt_type = self.cleaned_data.get("appt_type")

        # baseline appt_type must match arm
        community_appt_type = AppointmentType.objects.get(name=COMMUNITY)
        facility_appt_type = AppointmentType.objects.get(name=CLINIC)
        if is_baseline(instance=self.instance):
            if (
                appt_type == facility_appt_type
                and self.assignment_for_subject == COMMUNITY_ARM
            ):
                raise forms.ValidationError({"appt_type": _("Invalid. Expected community")})
            elif (
                appt_type == community_appt_type
                and self.assignment_for_subject == FACILITY_ARM
            ):
                raise forms.ValidationError({"appt_type": _("Invalid. Expected facility")})

        # community arm may not skip
        if (
            appt_status
            and appt_status == SKIPPED_APPT
            and self.assignment_for_subject == COMMUNITY_ARM
        ):
            raise forms.ValidationError(
                {"appt_status": _("Invalid. Community appointments may not be skipped")}
            )

        # ensure facility arm does not attend community
        if (
            appt_type
            and appt_type == AppointmentType.objects.get(name=COMMUNITY)
            and self.assignment_for_subject == FACILITY_ARM
        ):
            raise forms.ValidationError(
                {
                    "appt_type": _(
                        "Invalid. A facility-based participant may not attend in the community"
                    )
                }
            )
        return cleaned_data



from django.contrib.admin import SimpleListFilter
from edc_model_admin.list_filters import PastDateListFilter
from edc_visit_schedule.admin import (
    ScheduleStatusListFilter as BaseScheduleStatusListFilter,
)
from edc_visit_schedule.models import SubjectScheduleHistory


class VlListFilter(SimpleListFilter):
    title = "VL"
    parameter_name = "vl_value"

    def lookups(self, request, model_admin):
        return (
            ("missing", "Missing"),
            ("not_missing", "Not missing"),
        )

    def queryset(self, request, queryset):
        if self.value() != "none":
            if self.value() == "missing":
                queryset = queryset.filter(**{f"{self.parameter_name}__isnull": True})
            elif self.value() == "not_missing":
                queryset = queryset.filter(**{f"{self.parameter_name}__isnull": False})
        return queryset


class BaselineVlListFilter(VlListFilter):
    title = "Baseline VL"
    parameter_name = "baseline_vl"


class EndlineVlListFilter(VlListFilter):
    title = "Endline VL"
    parameter_name = "endline_vl"


class ScheduleStatusListFilter(BaseScheduleStatusListFilter):
    def lookups(self, request, model_admin):
        return ("on", "On schedule"), ("off", "Off schedule")

    @property
    def subject_identifiers(self):
        return SubjectScheduleHistory.objects.filter(
            offschedule_datetime__isnull=self.value() == "on"
        ).values_list("subject_identifier", flat=True)


class BaselineVlDateListFilter(PastDateListFilter):
    title = "Baseline VL Date"

    parameter_name = "baseline_vl_date"
    field_name = "baseline_vl_date"


class EndlineVlDateListFilter(PastDateListFilter):
    title = "Endline VL Date"

    parameter_name = "endline_vl_date"
    field_name = "endline_vl_date"


class LastVlDateListFilter(PastDateListFilter):
    title = "Last VL Date"

    parameter_name = "last_vl_date"
    field_name = "last_vl_date"


class NextVlDateListFilter(PastDateListFilter):
    title = "Next VL Date"

    parameter_name = "next_vl_date"
    field_name = "next_vl_date"


class VlDateListFilter(PastDateListFilter):
    title = "VL Date"

    parameter_name = "vl_date"
    field_name = "vl_date"



# Generated by Django 4.1.7 on 2023-04-17 23:14

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0028_remove_clinicalreviewbaseline_dm_test_and_more"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="historicaldminitialreview",
            name="med_start_estimated_date",
        ),
        migrations.AlterField(
            model_name="clinicalreviewbaseline",
            name="dm_dx",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="Has the patient ever been diagnosed with Diabetes",
            ),
        ),
        migrations.AlterField(
            model_name="clinicalreviewbaseline",
            name="hiv_dx",
            field=models.CharField(choices=[("Yes", "Yes"), ("No", "No")], max_length=15),
        ),
        migrations.AlterField(
            model_name="clinicalreviewbaseline",
            name="htn_dx",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="Has the patient ever been diagnosed with Hypertension",
            ),
        ),
        migrations.AlterField(
            model_name="historicalclinicalreviewbaseline",
            name="dm_dx",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="Has the patient ever been diagnosed with Diabetes",
            ),
        ),
        migrations.AlterField(
            model_name="historicalclinicalreviewbaseline",
            name="hiv_dx",
            field=models.CharField(choices=[("Yes", "Yes"), ("No", "No")], max_length=15),
        ),
        migrations.AlterField(
            model_name="historicalclinicalreviewbaseline",
            name="htn_dx",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="Has the patient ever been diagnosed with Hypertension",
            ),
        ),
    ]



# Generated by Django 4.2.10 on 2024-02-22 04:39

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_subject", "0143_historicalcareseekingb_historicalcareseekinga_and_more"),
    ]

    operations = [
        migrations.RenameField(
            model_name="careseekingb",
            old_name="meds_prescribed",
            new_name="med_prescribed",
        ),
        migrations.RenameField(
            model_name="historicalcareseekingb",
            old_name="meds_prescribed",
            new_name="med_prescribed",
        ),
    ]



# Generated by Django 4.2.11 on 2024-05-02 00:16

import _socket
from django.db import migrations, models
import django.db.models.deletion
import django.db.models.manager
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
import edc_sites.managers


class Migration(migrations.Migration):

    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        ("intecomm_subject", "0181_auto_20240422_1949"),
    ]

    operations = [
        migrations.CreateModel(
            name="DmCurrentConditions",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True, default=django_audit_fields.models.audit_model_mixin.utcnow
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                        verbose_name="Hostname created",
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                        verbose_name="Hostname modified",
                    ),
                ),
                (
                    "device_created",
                    models.CharField(blank=True, max_length=10, verbose_name="Device created"),
                ),
                (
                    "device_modified",
                    models.CharField(
                        blank=True, max_length=10, verbose_name="Device modified"
                    ),
                ),
                (
                    "locale_created",
                    models.CharField(
                        blank=True,
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        null=True,
                        verbose_name="Locale created",
                    ),
                ),
                (
                    "locale_modified",
                    models.CharField(
                        blank=True,
                        help_text="Auto-updated by Modeladmin",
                        max_length=10,
                        null=True,
                        verbose_name="Locale modified",
                    ),
                ),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50, unique=True)),
                ("baseline_date", models.DateField(null=True)),
                ("hiv", models.IntegerField(default=0)),
                ("htn", models.IntegerField(default=0)),
                ("dm", models.IntegerField(default=0)),
                ("hiv_dx_date", models.DateField(null=True)),
                ("htn_dx_date", models.DateField(null=True)),
                ("dm_dx_date", models.DateField(null=True)),
                (
                    "hiv_dx_days",
                    models.IntegerField(
                        default=None, help_text="Number of days relative to baseline"
                    ),
                ),
                (
                    "dm_dx_days",
                    models.BooleanField(
                        default=False, help_text="Number of days relative to baseline"
                    ),
                ),
                (
                    "htn_dx_days",
                    models.BooleanField(
                        default=False, help_text="Number of days relative to baseline"
                    ),
                ),
                ("comment", models.TextField(null=True)),
                (
                    "site",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
            ],
            options={
                "verbose_name": "Current Conditions",
                "verbose_name_plural": "Current Conditions",
                "abstract": False,
                "default_permissions": ("add", "change", "delete", "view", "export", "import"),
                "default_manager_name": "objects",
                "indexes": [
                    models.Index(
                        fields=["modified", "created"], name="intecomm_su_modifie_12b4ab_idx"
                    ),
                    models.Index(
                        fields=["user_modified", "user_created"],
                        name="intecomm_su_user_mo_596162_idx",
                    ),
                ],
            },
            managers=[
                ("on_site", edc_sites.managers.CurrentSiteManager()),
                ("objects", django.db.models.manager.Manager()),
            ],
        ),
    ]



from django.contrib import admin
from edc_adverse_event.modeladmin_mixins import HospitalizationModelAdminMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_sites.admin import SiteModelAdminMixin

from ..admin_site import intecomm_ae_admin
from ..forms import HospitalizationForm
from ..models import Hospitalization


@admin.register(Hospitalization, site=intecomm_ae_admin)
class HospitalizationAdmin(
    SiteModelAdminMixin, HospitalizationModelAdminMixin, SimpleHistoryAdmin
):
    form = HospitalizationForm

    # TODO: remove with Django > 4.2.5
    def get_list_filter(self, request) -> tuple[str]:
        list_filter = super().get_list_filter(request)
        self.list_filter = list_filter
        return list_filter



import numpy as np
import pandas as pd
from django.apps import apps as django_apps
from django_pandas.io import read_frame
from edc_utils import get_utcnow
from edc_visit_schedule.models import SubjectScheduleHistory

from ..models import Diagnoses as DiagnosesModel
from ..utils import update_diagnoses_model


class VlSummary:

    def __init__(
        self,
        baseline_months: int | None = None,
        endline_months: int | None = None,
        skip_update_dx: bool | None = None,
    ):
        if not skip_update_dx:
            update_diagnoses_model(delete_all=True)
        self._dfdx: pd.DataFrame = pd.DataFrame()
        self._df_initial_review: pd.DataFrame = pd.DataFrame()
        self._df_review: pd.DataFrame = pd.DataFrame()
        self.baseline_months = baseline_months or 3
        self.endline_months = endline_months or 9

    def to_dataframe(self) -> pd.DataFrame:
        """Prepare and return final df"""
        # merge dxdf with df1 (from hiv initial review)
        df = pd.merge(self.dfdx, self.df_initial_review, on="subject_identifier", how="left")
        # consider as baseline vl if 0m or earlier
        df["baseline_vl"] = df[
            df["baseline_date"] + pd.DateOffset(months=self.baseline_months)
            >= df["drawn_date"]
        ]["vl"]
        df["baseline_vl_date"] = df[
            df["baseline_date"] + pd.DateOffset(months=self.baseline_months)
            >= df["drawn_date"]
        ]["drawn_date"]

        # consider as endline vl if 6m or later
        df["endline_vl"] = df[
            df["drawn_date"] > df["baseline_date"] + pd.DateOffset(months=self.endline_months)
        ]["vl"]
        df["endline_vl_date"] = df[
            df["drawn_date"] > df["baseline_date"] + pd.DateOffset(months=self.endline_months)
        ]["drawn_date"]

        # merge df with vls from hiv review
        # this adds cols vl1, vl1_date, vl2, vl2_date
        df = pd.merge(df, self.df_review, on="subject_identifier", how="left")

        # null out vl1 and vl2 if same date as baseline_vl date
        df.loc[df.baseline_vl_date == df.vl1_date, ["vl1", "vl1_date"]] = [np.nan, pd.NaT]
        df.loc[df.baseline_vl_date == df.vl2_date, ["vl2", "vl2_date"]] = [np.nan, pd.NaT]

        # fill any missing baseline
        df[["baseline_vl", "baseline_vl_date"]] = df.apply(
            lambda row: self.get_best_baseline_vl_and_date(row, months=self.baseline_months),
            axis=1,
        ).to_list()

        # fill missing endlines
        df[["endline_vl", "endline_vl_date"]] = df.apply(
            lambda row: self.get_best_endline_vl_and_date(row, months=self.endline_months),
            axis=1,
        ).to_list()

        df["next_vl_date"] = df.apply(
            lambda row: (
                row["last_vl_date"] + pd.DateOffset(months=12)
                if pd.notna(row["last_vl_date"])
                else row["baseline_vl_date"] + pd.DateOffset(months=12)
            ),
            axis=1,
        )
        df["expected"] = df.apply(
            lambda row: self.get_expected_endline_vl_and_date(row), axis=1
        )

        df["offset"] = df["next_vl_date"].dt.to_period("M") - df[
            "offschedule_date"
        ].dt.to_period("M")
        df["offset"] = df["offset"].apply(lambda x: x.n if pd.notna(x) else np.nan)
        return df

    def to_model(self, model: str | None = None):
        df = self.to_dataframe()
        model_cls = django_apps.get_model(model or "intecomm_reports.vlsummary")
        model_cls.objects.all().delete()
        model_cls.objects.bulk_create(
            [
                model_cls(
                    subject_identifier=row["subject_identifier"],
                    site_id=row["site_id"],
                    baseline_date=(
                        None if pd.isna(row["baseline_date"]) else row["baseline_date"]
                    ),
                    baseline_vl=None if pd.isna(row["baseline_vl"]) else row["baseline_vl"],
                    baseline_vl_date=(
                        None if pd.isna(row["baseline_vl_date"]) else row["baseline_vl_date"]
                    ),
                    endline_vl=None if pd.isna(row["endline_vl"]) else row["endline_vl"],
                    endline_vl_date=(
                        None if pd.isna(row["endline_vl_date"]) else row["endline_vl_date"]
                    ),
                    offschedule_date=(
                        None if pd.isna(row["offschedule_date"]) else row["offschedule_date"]
                    ),
                    last_vl_date=(
                        None if pd.isna(row["last_vl_date"]) else row["last_vl_date"]
                    ),
                    next_vl_date=(
                        None if pd.isna(row["next_vl_date"]) else row["next_vl_date"]
                    ),
                    expected=row["expected"],
                    offset=None if pd.isna(row["offset"]) else row["offset"],
                )
                for _, row in df.iterrows()
            ]
        )

    @property
    def dfdx(self) -> pd.DataFrame:
        """DiagnosesModel with offschedule as df.
        (update this model first!)
        """
        if self._dfdx.empty:
            qs = (
                DiagnosesModel.objects.values(
                    "subject_identifier",
                    "site__id",
                    "hiv",
                    "hiv_dx_date",
                    "hiv_dx_days",
                    "baseline_date",
                )
                .filter(hiv=1)
                .order_by("subject_identifier")
            )
            dfdx = read_frame(qs)
            dfdx.rename(columns={"site__id": "site_id"}, inplace=True)
            dfdx["baseline_date"] = dfdx["baseline_date"].astype("datetime64[ns]")
            dfdx["hiv_dx_date"] = dfdx["hiv_dx_date"].astype("datetime64[ns]")
            # merge in offschedule
            qs = SubjectScheduleHistory.objects.all()
            dfoff = read_frame(qs, fieldnames=["subject_identifier", "offschedule_datetime"])
            dfoff["offschedule_date"] = dfoff["offschedule_datetime"].dt.date
            dfoff["offschedule_date"] = dfoff["offschedule_date"].astype("datetime64[ns]")
            dfoff.drop(columns=["offschedule_datetime"], inplace=True)
            dfdx = pd.merge(dfdx, dfoff, on="subject_identifier")
            dfdx = dfdx.sort_values(by=["subject_identifier"])
            dfdx = dfdx.reset_index(drop=True)
            self._dfdx = dfdx
        return self._dfdx

    @property
    def df_initial_review(self) -> pd.DataFrame:
        """HivInitialReview (df1).

        append empty cols to be populated
        """
        if self._df_initial_review.empty:
            model_cls = django_apps.get_model("intecomm_subject.hivinitialreview")
            df = self.get_review_model_as_df(model_cls)
            df["baseline_vl"] = np.nan
            df["baseline_vl_date"] = pd.NaT
            df["endline_vl"] = np.nan
            df["endline_vl_date"] = pd.NaT
            df = df.sort_values(by=["subject_identifier", "drawn_date"])
            df.reset_index(drop=True)
            self._df_initial_review = df
        return self._df_initial_review

    @property
    def df_review(self) -> pd.DataFrame:
        """HivReview (df1)"""
        if self._df_review.empty:
            model_cls = django_apps.get_model("intecomm_subject.hivreview")
            df = self.get_review_model_as_df(model_cls)
            df = df.sort_values(by=["subject_identifier", "drawn_date"])
            df.reset_index(drop=True)
            self._df_review = self.pivot_first_last_vl(df)
        return self._df_review

    @staticmethod
    def get_review_model_as_df(model_cls):
        qs = model_cls.objects.values(
            "subject_visit__subject_identifier",
            "has_vl",
            "vl",
            "drawn_date",
            "report_datetime",
        ).all()
        df = read_frame(
            qs,
            fieldnames=[
                "subject_visit__subject_identifier",
                "vl",
                "drawn_date",
                "report_datetime",
            ],
        )
        df.rename(
            columns={
                "subject_visit__subject_identifier": "subject_identifier",
                "site__id": "site_id",
            },
            inplace=True,
        )
        # vl column, replace nulls with NaN
        df.loc[df["vl"].isnull(), "vl"] = np.nan

        # report date
        df["report_date"] = pd.to_datetime(df["report_datetime"]).dt.date
        df["report_date"] = df["report_date"].astype("datetime64[ns]")
        df = df.drop("report_datetime", axis=1)

        # replace baseline drawn date with report date if drawn date is None
        df.loc[(df["drawn_date"].isnull()) & (df["vl"].notna()), "drawn_date"] = df[
            "report_date"
        ]
        df["drawn_date"] = df["drawn_date"].astype("datetime64[ns]")
        return df

    @staticmethod
    def pivot_first_last_vl(df: pd.DataFrame) -> pd.DataFrame:
        # get rows with non-null VL, select first drawn vl and last drawn vl
        # ... get rows with non-null VL
        dftmp = df[df.vl.notna()].reset_index(drop=True)
        # ... select first drawn vl
        df_first = (
            dftmp.sort_values(by="vl").groupby("subject_identifier", as_index=False).first()
        )
        df_first.drop(columns="report_date", inplace=True)
        df_first.rename(columns={"vl": "vl1", "drawn_date": "vl1_date"}, inplace=True)
        # ... select last drawn vl
        df_last = (
            dftmp.sort_values(by="vl").groupby("subject_identifier", as_index=False).last()
        )
        df_last.drop(columns="report_date", inplace=True)
        df_last.rename(columns={"vl": "vl2", "drawn_date": "vl2_date"}, inplace=True)
        df_last["last_vl"] = df_last["vl2"]
        df_last["last_vl_date"] = df_last["vl2_date"]
        # null out vl2 if same date as vl1
        df = pd.merge(df_first, df_last, on="subject_identifier", how="left")
        df.loc[df.vl1_date == df.vl2_date, ["vl2", "vl2_date"]] = [np.nan, pd.NaT]
        return df

    @staticmethod
    def get_expected_endline_vl_and_date(row) -> bool:
        value = True
        if pd.notna(row["next_vl_date"]) & pd.notna(row["offschedule_date"]):
            value = pd.Timestamp(row["next_vl_date"]) <= pd.Timestamp(
                row["offschedule_date"] + pd.DateOffset(months=3)
            )
        elif pd.notna(row["next_vl_date"]) & pd.isna(row["offschedule_date"]):
            value = row["next_vl_date"].date() <= get_utcnow().date()
        return value

    @staticmethod
    def get_best_baseline_vl_and_date(row, months: int = None) -> tuple[pd.Series, pd.Series]:
        values = row["baseline_vl"], row["baseline_vl_date"]
        if pd.isna(row["baseline_vl"]):
            if pd.notna(row["vl1"]) & pd.notna(row["vl2"]):
                if (
                    row["vl2_date"] < row["vl1_date"]
                    and row["baseline_date"] + pd.DateOffset(months=months) >= row["vl2_date"]
                ):
                    values = row["vl2"], row["vl2_date"]
                elif row["baseline_date"] + pd.DateOffset(months=months) >= row["vl1_date"]:
                    values = row["vl1"], row["vl1_date"]
            elif (
                pd.notna(row["vl1"])
                and row["baseline_date"] + pd.DateOffset(months=months) >= row["vl1_date"]
            ):
                values = row["vl1"], row["vl1_date"]
        return values

    @staticmethod
    def get_best_endline_vl_and_date(row, months: int = None) -> tuple[pd.Series, pd.Series]:
        values = row["endline_vl"], row["endline_vl_date"]
        if pd.isna(row["endline_vl"]):
            if pd.notna(row["vl1"]) & pd.notna(row["vl2"]):
                if (
                    row["vl2_date"] > row["vl1_date"]
                    and row["baseline_date"] + pd.DateOffset(months=months) < row["vl2_date"]
                ):
                    values = row["vl2"], row["vl2_date"]
                elif row["baseline_date"] + pd.DateOffset(months=months) < row["vl1_date"]:
                    values = row["vl1"], row["vl1_date"]
            elif (
                pd.notna(row["vl1"])
                and row["baseline_date"] + pd.DateOffset(months=months) < row["vl1_date"]
            ):
                values = row["vl1"], row["vl1_date"]
        return values



# Generated by Django 4.1.2 on 2022-11-25 23:45

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_screening",
            "0011_rename_hf_identifier_historicalpatientlog_hospital_identifier_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalpatientcall",
            name="last_attend_clinic",
            field=models.CharField(
                blank=True,
                help_text="This may be helpful when updating health records",
                max_length=25,
                null=True,
                verbose_name="Where did the patient last seek care",
            ),
        ),
        migrations.AlterField(
            model_name="patientcall",
            name="last_attend_clinic",
            field=models.CharField(
                blank=True,
                help_text="This may be helpful when updating health records",
                max_length=25,
                null=True,
                verbose_name="Where did the patient last seek care",
            ),
        ),
    ]



# Generated by Django 4.2.11 on 2024-07-29 21:04

from django.db import migrations, models
import django.db.models.deletion
import edc_sites.managers
import edc_utils.date


class Migration(migrations.Migration):

    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        ("intecomm_reports", "0008_alter_vl_options_alter_vlsummary_options_and_more"),
    ]

    operations = [
        migrations.CreateModel(
            name="MissingVlDrawDates",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=25)),
                (
                    "report_model",
                    models.CharField(
                        default="intecomm_reports.missingvldrawdates", max_length=50
                    ),
                ),
                ("created", models.DateTimeField(default=edc_utils.date.get_utcnow)),
                ("baseline_date", models.DateField(null=True)),
                ("visit_date", models.DateField(null=True)),
                ("visit_code", models.FloatField(null=True)),
                ("vl", models.IntegerField(null=True)),
                (
                    "site",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT, to="sites.site"
                    ),
                ),
            ],
            options={
                "verbose_name": "Viral load: Missing draw date",
                "verbose_name_plural": "Viral load: Missing draw dates",
                "db_table": "intecomm_reports_missingvldrawdates",
            },
            managers=[
                ("on_site", edc_sites.managers.CurrentSiteManager()),
            ],
        ),
    ]



from django import forms
from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import gettext as _
from edc_constants.constants import BOTH, NO, NOT_APPLICABLE, OUTPATIENT, YES
from edc_crf.crf_form_validator_mixins import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfSingletonModelFormMixin
from edc_dx import Diagnoses, get_diagnosis_labels
from edc_form_validators import INVALID_ERROR, FormValidator

from ...choices import ACCOMPANIED_BY, TRAVEL_METHODS
from ...constants import ALONE
from ...models import CareseekingA, CareseekingB
from ..mixins import CrfModelFormMixin


class CareseekingBFormValidator(CrfFormValidatorMixin, FormValidator):
    def clean(self) -> None:
        try:
            CareseekingA.objects.get(subject_visit__subject_identifier=self.subject_identifier)
        except ObjectDoesNotExist:
            raise forms.ValidationError(
                f"Please complete form {CareseekingA._meta.verbose_name} first."
            )
        care_seeking_b = CareseekingB.objects.filter(
            subject_visit__subject_identifier=self.related_visit.subject_identifier
        ).exclude(subject_visit_id=self.related_visit.id)
        if care_seeking_b.count() > 0:
            raise forms.ValidationError(
                f"This form has already been submitted. See {care_seeking_b[0].subject_visit}."
            )

        # B1:
        self.applicable_if(YES, field="needed_care", field_applicable="accessed_care")
        self.applicable_if(NO, field="accessed_care", field_applicable="no_accessed_care")
        self.validate_other_specify(
            field="no_accessed_care", other_specify_field="no_accessed_care_other"
        )
        self.applicable_if(YES, field="accessed_care", field_applicable="seek_facility")
        self.validate_other_specify(
            field="accessed_care", other_specify_field="accessed_care_other"
        )
        self.applicable_if(YES, field="accessed_care", field_applicable="care_type")
        self.required_if(
            OUTPATIENT,
            BOTH,
            field="care_type",
            field_required="outpatient_visits",
            field_required_evaluate_as_int=True,
        )

        # B2: Travel and expenses at last/most recent visit
        self.m2m_required_if(YES, field="accessed_care", m2m_field="travel_method")
        self.required_if_m2m(
            *[t[0] for t in TRAVEL_METHODS],
            field="travel_method",
            field_required="travel_duration",
            field_other_evaluate_as_int=True,
        )
        self.required_if_m2m(
            *[t[0] for t in TRAVEL_METHODS],
            field="travel_method",
            field_required="travel_costs",
            field_other_evaluate_as_int=True,
        )
        self.required_if_m2m(
            *[t[0] for t in TRAVEL_METHODS],
            field="travel_method",
            field_required="food_costs",
            field_other_evaluate_as_int=True,
        )
        self.required_if(
            YES,
            field="accessed_care",
            field_required="care_costs",
            field_required_evaluate_as_int=True,
        )

        # B3: Medications at last/most recent visit
        self.applicable_if(YES, field="accessed_care", field_applicable="med_prescribed")
        self.m2m_required_if(YES, field="med_prescribed", m2m_field="med_conditions")
        med_conditions = self.cleaned_data.get("med_conditions")
        diagnoses = Diagnoses(
            subject_identifier=self.subject_identifier,
            report_datetime=self.report_datetime,
            lte=True,
        )
        if invalid_conditions := set([o.name.lower() for o in med_conditions or []]) - set(
            diagnoses.initial_reviews
        ):
            if labels := ", ".join(
                [
                    get_diagnosis_labels().get(label)
                    for label in invalid_conditions
                    if label in get_diagnosis_labels()
                ]
            ):
                self.raise_validation_error(
                    {
                        "med_conditions": (
                            f"Subject has not been diagnosed with these conditions: {labels}"
                        )
                    },
                    INVALID_ERROR,
                )

        self.m2m_other_specify(m2m_field="med_conditions", field_other="med_conditions_other")
        self.applicable_if(YES, field="med_prescribed", field_applicable="med_collected")
        self.applicable_if(
            NO, field="med_collected", field_applicable="med_not_collected_reason"
        )
        self.validate_other_specify(
            field="med_not_collected_reason",
            other_specify_field="med_not_collected_reason_other",
        )
        self.required_if(
            YES,
            field="med_collected",
            field_required="med_cost_tot",
            field_required_evaluate_as_int=True,
        )

        # B4: Tests at last/most recent visit
        self.applicable_if(YES, field="accessed_care", field_applicable="tests_requested")
        self.applicable_if(YES, field="tests_requested", field_applicable="tests_done")
        self.required_if(NO, field="tests_done", field_required="tests_not_done_reason")
        self.validate_other_specify(
            field="tests_not_done_reason", other_specify_field="tests_not_done_other"
        )
        self.required_if(
            YES,
            field="tests_done",
            field_required="tests_cost",
            field_required_evaluate_as_int=True,
        )

        # B5: Time and expenses at last/most recent visit
        self.required_if(YES, field="accessed_care", field_required="care_visit_duration")
        self.applicable_if(YES, field="accessed_care", field_applicable="missed_activities")
        self.validate_other_specify(
            field="missed_activities", other_specify_field="missed_activities_other"
        )

        # B6: People who accompanied you to last/most recent visit
        self.applicable_if(YES, field="accessed_care", field_applicable="accompany")
        self.required_if(
            *[t[0] for t in ACCOMPANIED_BY if t[0] not in [ALONE, NOT_APPLICABLE]],
            field="accompany",
            field_required="accompany_num",
        )
        self.applicable_if(
            *[t[0] for t in ACCOMPANIED_BY if t[0] not in [ALONE, NOT_APPLICABLE]],
            field="accompany",
            field_applicable="accompany_wait",
        )
        self.applicable_if(
            *[t[0] for t in ACCOMPANIED_BY if t[0] not in [ALONE, NOT_APPLICABLE]],
            field="accompany",
            field_applicable="accompany_alt",
        )
        self.validate_other_specify(
            field="accompany_alt", other_specify_field="accompany_alt_other"
        )

        # B7: Your expenses in the past 3 months
        self.m2m_required_if(YES, field="accessed_care", m2m_field="money_sources")
        money_sources = [o.name for o in self.cleaned_data.get("money_sources")]
        if len(money_sources) > 3:
            self.raise_validation_error(
                {"money_sources": "Please limit to no more than 3 selections"}, INVALID_ERROR
            )
        self.m2m_other_specify(m2m_field="money_sources", field_other="money_sources_other")
        self.applicable_if(YES, field="accessed_care", field_applicable="money_source_main")
        if (
            self.cleaned_data.get("money_source_main")
            and money_sources
            and self.cleaned_data.get("money_source_main") not in money_sources
        ):
            self.raise_validation_error(
                {"money_source_main": _("Response not found among responses given above")},
                INVALID_ERROR,
            )
        # B8: About your inpatient visit
        self.required_if(YES, field="inpatient", field_required="inpatient_days")
        self.m2m_required_if(YES, field="inpatient", m2m_field="inpatient_reasons")
        self.m2m_other_specify(
            m2m_field="inpatient_reasons", field_other="inpatient_reasons_other"
        )

        # B9: Inpatient visit: expenses
        self.required_if(
            YES,
            field="inpatient",
            field_required="inpatient_cost",
            field_required_evaluate_as_int=True,
        )
        self.required_if(YES, field="inpatient", field_required="inpatient_accompany")
        self.required_if(YES, field="inpatient", field_required="inpatient_food")
        self.required_if(
            YES,
            field="inpatient_food",
            field_required="inpatient_food_cost",
            field_required_evaluate_as_int=True,
        )
        self.required_if(
            YES,
            field="inpatient",
            field_required="inpatient_nowork_days",
            field_required_evaluate_as_int=True,
        )
        self.required_if(YES, field="inpatient", field_required="inpatient_household_nowork")
        self.required_if(
            YES,
            field="inpatient_household_nowork",
            field_required="inpatient_household_nowork_days",
            field_required_evaluate_as_int=True,
        )

        # B10: Inpatient visit: sources of payment for expenses
        self.m2m_required_if(YES, field="inpatient", m2m_field="inpatient_money_sources")
        inpatient_money_sources = [
            o.name for o in self.cleaned_data.get("inpatient_money_sources")
        ]
        if len(inpatient_money_sources) > 3:
            self.raise_validation_error(
                {"inpatient_money_sources": "Please limit to no more than 3 selections"},
                INVALID_ERROR,
            )
        self.m2m_other_specify(
            m2m_field="inpatient_money_sources", field_other="inpatient_money_sources_other"
        )
        self.applicable_if(
            YES, field="inpatient", field_applicable="inpatient_money_sources_main"
        )
        if (
            self.cleaned_data.get("inpatient_money_sources_main")
            and inpatient_money_sources
            and self.cleaned_data.get("inpatient_money_sources_main")
            not in [o.name for o in self.cleaned_data.get("inpatient_money_sources")]
        ):
            self.raise_validation_error(
                {
                    "inpatient_money_sources_main": _(
                        "Response not found among responses given above"
                    )
                },
                INVALID_ERROR,
            )
        self.validate_other_specify(
            field="inpatient_money_sources_main",
            other_specify_field="inpatient_money_sources_main_other",
        )


class CareseekingBForm(
    CrfSingletonModelFormMixin,
    CrfModelFormMixin,
    forms.ModelForm,
):
    form_validator_cls = CareseekingBFormValidator

    class Meta:
        model = CareseekingB
        fields = "__all__"



from django.db import models
from edc_constants.choices import YES_NO
from edc_constants.constants import NO
from edc_model.models import BaseUuidModel

from ..model_mixins import CrfModelMixin


class ComplicationsFollowup(CrfModelMixin, BaseUuidModel):
    # stroke
    stroke = models.CharField(verbose_name="Stroke", max_length=25, choices=YES_NO)

    stroke_date = models.DateField(
        verbose_name="If yes, date",
        null=True,
        blank=True,
        help_text="If exact date not known, see SOP on how to estimate a date.",
    )

    # heart_attack
    heart_attack = models.CharField(
        verbose_name="Heart attack / heart failure",
        max_length=25,
        choices=YES_NO,
    )

    heart_attack_date = models.DateField(
        verbose_name="If yes, date",
        null=True,
        blank=True,
        help_text="If exact date not known, see SOP on how to estimate a date.",
    )

    # renal
    renal_disease = models.CharField(
        verbose_name="Renal (kidney) disease",
        max_length=25,
        choices=YES_NO,
    )

    renal_disease_date = models.DateField(
        verbose_name="If yes, date",
        null=True,
        blank=True,
        help_text="If exact date not known, see SOP on how to estimate a date.",
    )
    # vision
    vision = models.CharField(
        verbose_name="Vision problems (e.g. blurred vision)",
        max_length=25,
        choices=YES_NO,
    )

    vision_date = models.DateField(
        verbose_name="If yes, date",
        null=True,
        blank=True,
        help_text="If exact date not known, see SOP on how to estimate a date.",
    )

    # numbness
    numbness = models.CharField(
        verbose_name="Numbness / burning sensation",
        max_length=25,
        choices=YES_NO,
    )

    numbness_date = models.DateField(
        verbose_name="If yes, date",
        null=True,
        blank=True,
        help_text="If exact date not known, see SOP on how to estimate a date.",
    )

    # foot ulcers
    foot_ulcers = models.CharField(
        verbose_name="Foot ulcers",
        max_length=25,
        choices=YES_NO,
    )

    foot_ulcers_date = models.DateField(
        verbose_name="If yes, date",
        null=True,
        blank=True,
        help_text="If exact date not known, see SOP on how to estimate a date.",
    )
    #
    complications = models.CharField(
        verbose_name="Are there any other major complications to report?",
        max_length=25,
        choices=YES_NO,
        default=NO,
    )

    complications_other = models.TextField(
        null=True,
        blank=True,
        help_text="Please include dates",
    )

    class Meta(CrfModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Complications: Followup"
        verbose_name_plural = "Complications: Followup"
        indexes = CrfModelMixin.Meta.indexes + BaseUuidModel.Meta.indexes



from django.contrib import admin
from edc_action_item import action_fieldset_tuple
from edc_lab_results.admin import BloodResultsModelAdminMixin
from edc_lab_results.fieldsets import BloodResultFieldset

from ...admin_site import intecomm_subject_admin
from ...forms import BloodResultsFbcForm
from ...models import BloodResultsFbc
from ..modeladmin_mixins import CrfModelAdmin


@admin.register(BloodResultsFbc, site=intecomm_subject_admin)
class BloodResultsFbcAdmin(BloodResultsModelAdminMixin, CrfModelAdmin):
    form = BloodResultsFbcForm
    fieldsets = BloodResultFieldset(
        BloodResultsFbc.lab_panel,
        model_cls=BloodResultsFbc,
        extra_fieldsets=[(-1, action_fieldset_tuple)],
    ).fieldsets



# Generated by Django 4.2.3 on 2023-08-10 03:27

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_subject",
            "0101_alter_healtheconomicspatient_pat_education_old_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_education_old",
            field=models.CharField(
                default="QUESTION_RETIRED",
                editable=False,
                max_length=25,
                verbose_name="Highest level of education completed by the household head?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_employment_type_old",
            field=models.CharField(
                default="QUESTION_RETIRED",
                editable=False,
                max_length=25,
                verbose_name="Household head’s type of employment",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_ethnicity_old",
            field=models.CharField(
                default="QUESTION_RETIRED",
                editable=False,
                max_length=25,
                verbose_name="What is the household head’s ethnic background?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_religion_old",
            field=models.CharField(
                default="QUESTION_RETIRED",
                editable=False,
                max_length=25,
                verbose_name="How would you describe the household head’s religious orientation?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_education_old",
            field=models.CharField(
                default="QUESTION_RETIRED",
                editable=False,
                max_length=25,
                verbose_name="Highest level of education completed by the household head?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_employment_type_old",
            field=models.CharField(
                default="QUESTION_RETIRED",
                editable=False,
                max_length=25,
                verbose_name="Household head’s type of employment",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_ethnicity_old",
            field=models.CharField(
                default="QUESTION_RETIRED",
                editable=False,
                max_length=25,
                verbose_name="What is the household head’s ethnic background?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_religion_old",
            field=models.CharField(
                default="QUESTION_RETIRED",
                editable=False,
                max_length=25,
                verbose_name="How would you describe the household head’s religious orientation?",
            ),
        ),
    ]



# Generated by Django 5.0.8 on 2024-08-26 14:55

import django.db.models.deletion
import edc_utils.date
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "intecomm_reports",
            "0011_alter_missingvldrawdates_site_alter_vlsummary_site_and_more",
        ),
        ("sites", "0002_alter_domain_unique"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="vlsummary2",
            name="site",
        ),
        migrations.AlterModelManagers(
            name="diagnoses",
            managers=[],
        ),
        migrations.AlterModelManagers(
            name="missingvldrawdates",
            managers=[],
        ),
        migrations.AddField(
            model_name="diagnoses",
            name="report_model",
            field=models.CharField(default="meta_reports.diagnoses", max_length=50),
        ),
        migrations.AlterField(
            model_name="diagnoses",
            name="created",
            field=models.DateTimeField(),
        ),
        migrations.AlterField(
            model_name="diagnoses",
            name="site",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.DO_NOTHING, to="sites.site"
            ),
        ),
        migrations.CreateModel(
            name="VlSummary6m",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50, unique=True)),
                ("created", models.DateTimeField(default=edc_utils.date.get_utcnow)),
                ("baseline_date", models.DateField(null=True)),
                ("baseline_vl_date", models.DateField(null=True)),
                ("endline_vl_date", models.DateField(null=True)),
                ("baseline_vl", models.IntegerField(null=True)),
                ("endline_vl", models.IntegerField(null=True)),
                ("offschedule_date", models.DateField(null=True)),
                ("last_vl_date", models.DateField(null=True)),
                ("next_vl_date", models.DateField(null=True)),
                ("expected", models.BooleanField(null=True)),
                ("offset", models.IntegerField(null=True)),
                (
                    "report_model",
                    models.CharField(default="intecomm_reports.vlsummary9m", max_length=50),
                ),
                (
                    "site",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.DO_NOTHING, to="sites.site"
                    ),
                ),
            ],
            options={
                "verbose_name": "Viral load summary (endline >= 6m)",
                "verbose_name_plural": "Viral load summary (endline >= 6m)",
            },
        ),
        migrations.CreateModel(
            name="VlSummary9m",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50, unique=True)),
                ("created", models.DateTimeField(default=edc_utils.date.get_utcnow)),
                ("baseline_date", models.DateField(null=True)),
                ("baseline_vl_date", models.DateField(null=True)),
                ("endline_vl_date", models.DateField(null=True)),
                ("baseline_vl", models.IntegerField(null=True)),
                ("endline_vl", models.IntegerField(null=True)),
                ("offschedule_date", models.DateField(null=True)),
                ("last_vl_date", models.DateField(null=True)),
                ("next_vl_date", models.DateField(null=True)),
                ("expected", models.BooleanField(null=True)),
                ("offset", models.IntegerField(null=True)),
                (
                    "report_model",
                    models.CharField(default="intecomm_reports.vlsummary6m", max_length=50),
                ),
                (
                    "site",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.DO_NOTHING, to="sites.site"
                    ),
                ),
            ],
            options={
                "verbose_name": "Viral load summary (endline >= 9m)",
                "verbose_name_plural": "Viral load summary (endline >= 9m)",
            },
        ),
        migrations.DeleteModel(
            name="VlSummary",
        ),
        migrations.DeleteModel(
            name="VlSummary2",
        ),
    ]



# Generated by Django 5.0.3 on 2024-03-19 04:31

import django_crypto_fields.fields.encrypted_char_field
import edc_consent.validators
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        (
            "intecomm_consent",
            "0024_rename_updates_versions_historicalsubjectconsent_update_versions_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalsubjectconsent",
            name="guardian_name",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text="Required only if participant is a minor.<BR>Format is 'LASTNAME, FIRSTNAME'. All uppercase separated by a comma. (Encryption: RSA local)",
                max_length=71,
                null=True,
                validators=[edc_consent.validators.FullNameValidator()],
                verbose_name="Guardian's last and first name",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectconsenttz",
            name="guardian_name",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text="Required only if participant is a minor.<BR>Format is 'LASTNAME, FIRSTNAME'. All uppercase separated by a comma. (Encryption: RSA local)",
                max_length=71,
                null=True,
                validators=[edc_consent.validators.FullNameValidator()],
                verbose_name="Guardian's last and first name",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectconsentug",
            name="guardian_name",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text="Required only if participant is a minor.<BR>Format is 'LASTNAME, FIRSTNAME'. All uppercase separated by a comma. (Encryption: RSA local)",
                max_length=71,
                null=True,
                validators=[edc_consent.validators.FullNameValidator()],
                verbose_name="Guardian's last and first name",
            ),
        ),
        migrations.AlterField(
            model_name="subjectconsent",
            name="guardian_name",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text="Required only if participant is a minor.<BR>Format is 'LASTNAME, FIRSTNAME'. All uppercase separated by a comma. (Encryption: RSA local)",
                max_length=71,
                null=True,
                validators=[edc_consent.validators.FullNameValidator()],
                verbose_name="Guardian's last and first name",
            ),
        ),
    ]



from django.contrib import admin
from django_audit_fields import audit_fieldset_tuple
from django_revision.modeladmin_mixin import ModelAdminRevisionMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_model_admin.mixins import (
    ModelAdminFormAutoNumberMixin,
    ModelAdminFormInstructionsMixin,
    ModelAdminInstitutionMixin,
    TemplatesModelAdminMixin,
)
from edc_sites.admin import SiteModelAdminMixin

from ..admin_site import intecomm_group_admin
from ..forms import PatientGroupMeetingForm
from ..models import PatientGroupMeeting


@admin.register(PatientGroupMeeting, site=intecomm_group_admin)
class PatientGroupMeetingAdmin(
    SiteModelAdminMixin,
    TemplatesModelAdminMixin,
    ModelAdminFormInstructionsMixin,
    ModelAdminFormAutoNumberMixin,
    ModelAdminRevisionMixin,
    ModelAdminInstitutionMixin,
    SimpleHistoryAdmin,
):
    form = PatientGroupMeetingForm

    show_object_tools = True
    change_list_template: str = "intecomm_group/admin/patientgroupmeeting_change_list.html"

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "report_datetime",
                    "patient_group_appointment",
                    "meeting_datetime",
                )
            },
        ),
        (
            "Attendance",
            {"fields": ("patients",)},
        ),
        (
            "Notes",
            {"fields": ("notes",)},
        ),
        audit_fieldset_tuple,
    )

    filter_horizontal = ("patients",)

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        db = kwargs.get("using")
        if db_field.name == "patient_group_appointment" and request.GET.get(
            "patient_group_appointment"
        ):
            kwargs["queryset"] = db_field.related_model._default_manager.using(db).filter(
                pk=request.GET.get("patient_group_appointment")
            )
        else:
            kwargs["queryset"] = db_field.related_model._default_manager.none()
        return super().formfield_for_foreignkey(db_field, request, **kwargs)



from django import forms
from intecomm_form_validators.subject import DmInitialReviewFormValidator

from ..models import DmInitialReview
from .mixins import CrfModelFormMixin


class DmInitialReviewForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = DmInitialReviewFormValidator

    class Meta:
        model = DmInitialReview
        fields = "__all__"



# Generated by Django 4.2.11 on 2024-04-22 16:44

import edc_model.models.fields.other_charfield
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_lists", "0012_visitreasons_custom_name"),
        ("intecomm_subject", "0178_rename_travel_time_careseekinga_travel_duration_and_more"),
    ]

    operations = [
        migrations.RenameField(
            model_name="historicalhtnreview",
            old_name="managed_by",
            new_name="managed_by_old",
        ),
        migrations.RenameField(
            model_name="htnreview",
            old_name="managed_by",
            new_name="managed_by_old",
        ),
    ]



# Generated by Django 4.2.7 on 2023-12-05 20:46

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0132_alter_subjectrequisition_unique_together_and_more"),
    ]

    operations = [
        migrations.AddIndex(
            model_name="bloodresultsfbc",
            index=models.Index(
                fields=["subject_visit", "site"], name="intecomm_su_subject_3bd165_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultsfbc",
            index=models.Index(
                fields=["modified", "created"], name="intecomm_su_modifie_4b24dd_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultsfbc",
            index=models.Index(
                fields=["user_modified", "user_created"], name="intecomm_su_user_mo_263115_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultsglu",
            index=models.Index(
                fields=["subject_visit", "site"], name="intecomm_su_subject_eb429a_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultsglu",
            index=models.Index(
                fields=["modified", "created"], name="intecomm_su_modifie_ac7e13_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultsglu",
            index=models.Index(
                fields=["user_modified", "user_created"], name="intecomm_su_user_mo_e3b982_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultshba1c",
            index=models.Index(
                fields=["subject_visit", "site"], name="intecomm_su_subject_2983d9_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultshba1c",
            index=models.Index(
                fields=["modified", "created"], name="intecomm_su_modifie_b05d6d_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultshba1c",
            index=models.Index(
                fields=["user_modified", "user_created"], name="intecomm_su_user_mo_c6051a_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultsins",
            index=models.Index(
                fields=["subject_visit", "site"], name="intecomm_su_subject_01a47a_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultsins",
            index=models.Index(
                fields=["modified", "created"], name="intecomm_su_modifie_341708_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultsins",
            index=models.Index(
                fields=["user_modified", "user_created"], name="intecomm_su_user_mo_dffdda_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultslft",
            index=models.Index(
                fields=["subject_visit", "site"], name="intecomm_su_subject_df2b7f_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultslft",
            index=models.Index(
                fields=["modified", "created"], name="intecomm_su_modifie_18c09e_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultslft",
            index=models.Index(
                fields=["user_modified", "user_created"], name="intecomm_su_user_mo_b12c36_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultslipid",
            index=models.Index(
                fields=["subject_visit", "site"], name="intecomm_su_subject_0dd0d1_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultslipid",
            index=models.Index(
                fields=["modified", "created"], name="intecomm_su_modifie_2deb12_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultslipid",
            index=models.Index(
                fields=["user_modified", "user_created"], name="intecomm_su_user_mo_4870e2_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultsrft",
            index=models.Index(
                fields=["subject_visit", "site"], name="intecomm_su_subject_7bc0b6_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultsrft",
            index=models.Index(
                fields=["modified", "created"], name="intecomm_su_modifie_c1e6b4_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="bloodresultsrft",
            index=models.Index(
                fields=["user_modified", "user_created"], name="intecomm_su_user_mo_3922a4_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="cd4result",
            index=models.Index(
                fields=["subject_visit", "site"], name="intecomm_su_subject_a40fc4_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="cd4result",
            index=models.Index(
                fields=["subject_visit", "report_datetime"],
                name="intecomm_su_subject_85e709_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="cd4result",
            index=models.Index(
                fields=["modified", "created"], name="intecomm_su_modifie_9f8432_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="cd4result",
            index=models.Index(
                fields=["user_modified", "user_created"], name="intecomm_su_user_mo_a8ac98_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="clinicalnote",
            index=models.Index(
                fields=["subject_visit", "site"], name="intecomm_su_subject_193cc0_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="clinicalnote",
            index=models.Index(
                fields=["modified", "created"], name="intecomm_su_modifie_1e0564_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="clinicalnote",
            index=models.Index(
                fields=["user_modified", "user_created"], name="intecomm_su_user_mo_81a993_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="clinicalreview",
            index=models.Index(
                fields=["subject_visit", "site"], name="intecomm_su_subject_c37967_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="clinicalreview",
            index=models.Index(
                fields=["modified", "created"], name="intecomm_su_modifie_e40016_idx"
            ),
        ),
        migrations.AddIndex(
            model_name="clinicalreview",
            index=models.Index(
                fields=["user_modified", "user_created"], name="intecomm_su_user_mo_3f082b_idx"
            ),
        ),
    ]



# Generated by Django 4.2.1 on 2023-05-24 22:33

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        (
            "intecomm_subject",
            "0049_alter_healtheconomicsincome_external_remittance_value_and_more",
        ),
    ]

    operations = [
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_ethnicity",
            field=models.CharField(
                choices=[
                    ("baganda", "Baganda"),
                    ("banyakole", "Banyakole"),
                    ("basoga", "Basoga"),
                    ("bakiga", "Bakiga"),
                    ("iteso", "Iteso"),
                    ("langi", "Langi"),
                    ("acholi", "Acholi"),
                    ("bagisu", "Bagisu"),
                    ("lugbara", "Lugbara"),
                    ("banyoro", "Banyoro"),
                    ("bakhonzo", "Bakhonzo"),
                    ("batoro", "Batoro"),
                    ("alur", "Alur"),
                    ("OTHER", "Other, specify ..."),
                    ("msukuma", "Msukuma"),
                    ("mjita", "Mjita"),
                    ("mzinza", "Mzinza"),
                    ("myiramba", "Myiramba"),
                    ("mkara_mkerewe", "Mkara/Mkerewe"),
                    ("mhaya", "Mhaya"),
                    ("mjaluo", "Mjaluo"),
                    ("mkuria_mshashi", "Mkuria/Mshashi"),
                    ("mchaga", "Mchaga"),
                    ("mhindi", "Mhindi"),
                    ("mwarabu", "Mwarabu"),
                    ("OTHER", "Other, specify ..."),
                ],
                max_length=25,
                verbose_name="What is the household head’s ethnic background?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_religion",
            field=models.CharField(
                choices=[
                    ("catholic", "Catholic"),
                    ("protestant", "Protestant"),
                    ("muslim", "Muslim"),
                    ("pentecostal", "Pentecostal"),
                    ("seventh_day_adventist", "Seventh day Adventist"),
                    ("no_religion", "No religion"),
                    ("OTHER", "Other, specify ..."),
                    ("muslim", "Muslim"),
                    ("seventh_day_adventist", "Seventh day Adventist"),
                    ("tz_assemblies_of_god", "Tanzanian Assemblies of God"),
                    ("catholic", "Catholic"),
                    ("lutheran", "Lutheran"),
                    ("moravian", "Moravian"),
                    ("pentecostal", "Pentecostal"),
                    ("african_inland_church", "African Inland Church"),
                    ("no_religion", "No religion"),
                    ("OTHER", "Other, specify ..."),
                ],
                max_length=25,
                verbose_name="How would you describe the household head’s religious orientation?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicspatient",
            name="pat_ethnicity",
            field=models.CharField(
                choices=[
                    ("baganda", "Baganda"),
                    ("banyakole", "Banyakole"),
                    ("basoga", "Basoga"),
                    ("bakiga", "Bakiga"),
                    ("iteso", "Iteso"),
                    ("langi", "Langi"),
                    ("acholi", "Acholi"),
                    ("bagisu", "Bagisu"),
                    ("lugbara", "Lugbara"),
                    ("banyoro", "Banyoro"),
                    ("bakhonzo", "Bakhonzo"),
                    ("batoro", "Batoro"),
                    ("alur", "Alur"),
                    ("OTHER", "Other, specify ..."),
                    ("msukuma", "Msukuma"),
                    ("mjita", "Mjita"),
                    ("mzinza", "Mzinza"),
                    ("myiramba", "Myiramba"),
                    ("mkara_mkerewe", "Mkara/Mkerewe"),
                    ("mhaya", "Mhaya"),
                    ("mjaluo", "Mjaluo"),
                    ("mkuria_mshashi", "Mkuria/Mshashi"),
                    ("mchaga", "Mchaga"),
                    ("mhindi", "Mhindi"),
                    ("mwarabu", "Mwarabu"),
                    ("OTHER", "Other, specify ..."),
                ],
                max_length=25,
                verbose_name="What is the household head’s ethnic background?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicspatient",
            name="pat_religion",
            field=models.CharField(
                choices=[
                    ("catholic", "Catholic"),
                    ("protestant", "Protestant"),
                    ("muslim", "Muslim"),
                    ("pentecostal", "Pentecostal"),
                    ("seventh_day_adventist", "Seventh day Adventist"),
                    ("no_religion", "No religion"),
                    ("OTHER", "Other, specify ..."),
                    ("muslim", "Muslim"),
                    ("seventh_day_adventist", "Seventh day Adventist"),
                    ("tz_assemblies_of_god", "Tanzanian Assemblies of God"),
                    ("catholic", "Catholic"),
                    ("lutheran", "Lutheran"),
                    ("moravian", "Moravian"),
                    ("pentecostal", "Pentecostal"),
                    ("african_inland_church", "African Inland Church"),
                    ("no_religion", "No religion"),
                    ("OTHER", "Other, specify ..."),
                ],
                max_length=25,
                verbose_name="How would you describe the household head’s religious orientation?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_ethnicity",
            field=models.CharField(
                choices=[
                    ("baganda", "Baganda"),
                    ("banyakole", "Banyakole"),
                    ("basoga", "Basoga"),
                    ("bakiga", "Bakiga"),
                    ("iteso", "Iteso"),
                    ("langi", "Langi"),
                    ("acholi", "Acholi"),
                    ("bagisu", "Bagisu"),
                    ("lugbara", "Lugbara"),
                    ("banyoro", "Banyoro"),
                    ("bakhonzo", "Bakhonzo"),
                    ("batoro", "Batoro"),
                    ("alur", "Alur"),
                    ("OTHER", "Other, specify ..."),
                    ("msukuma", "Msukuma"),
                    ("mjita", "Mjita"),
                    ("mzinza", "Mzinza"),
                    ("myiramba", "Myiramba"),
                    ("mkara_mkerewe", "Mkara/Mkerewe"),
                    ("mhaya", "Mhaya"),
                    ("mjaluo", "Mjaluo"),
                    ("mkuria_mshashi", "Mkuria/Mshashi"),
                    ("mchaga", "Mchaga"),
                    ("mhindi", "Mhindi"),
                    ("mwarabu", "Mwarabu"),
                    ("OTHER", "Other, specify ..."),
                ],
                max_length=25,
                verbose_name="What is the household head’s ethnic background?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_religion",
            field=models.CharField(
                choices=[
                    ("catholic", "Catholic"),
                    ("protestant", "Protestant"),
                    ("muslim", "Muslim"),
                    ("pentecostal", "Pentecostal"),
                    ("seventh_day_adventist", "Seventh day Adventist"),
                    ("no_religion", "No religion"),
                    ("OTHER", "Other, specify ..."),
                    ("muslim", "Muslim"),
                    ("seventh_day_adventist", "Seventh day Adventist"),
                    ("tz_assemblies_of_god", "Tanzanian Assemblies of God"),
                    ("catholic", "Catholic"),
                    ("lutheran", "Lutheran"),
                    ("moravian", "Moravian"),
                    ("pentecostal", "Pentecostal"),
                    ("african_inland_church", "African Inland Church"),
                    ("no_religion", "No religion"),
                    ("OTHER", "Other, specify ..."),
                ],
                max_length=25,
                verbose_name="How would you describe the household head’s religious orientation?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicspatient",
            name="pat_ethnicity",
            field=models.CharField(
                choices=[
                    ("baganda", "Baganda"),
                    ("banyakole", "Banyakole"),
                    ("basoga", "Basoga"),
                    ("bakiga", "Bakiga"),
                    ("iteso", "Iteso"),
                    ("langi", "Langi"),
                    ("acholi", "Acholi"),
                    ("bagisu", "Bagisu"),
                    ("lugbara", "Lugbara"),
                    ("banyoro", "Banyoro"),
                    ("bakhonzo", "Bakhonzo"),
                    ("batoro", "Batoro"),
                    ("alur", "Alur"),
                    ("OTHER", "Other, specify ..."),
                    ("msukuma", "Msukuma"),
                    ("mjita", "Mjita"),
                    ("mzinza", "Mzinza"),
                    ("myiramba", "Myiramba"),
                    ("mkara_mkerewe", "Mkara/Mkerewe"),
                    ("mhaya", "Mhaya"),
                    ("mjaluo", "Mjaluo"),
                    ("mkuria_mshashi", "Mkuria/Mshashi"),
                    ("mchaga", "Mchaga"),
                    ("mhindi", "Mhindi"),
                    ("mwarabu", "Mwarabu"),
                    ("OTHER", "Other, specify ..."),
                ],
                max_length=25,
                verbose_name="What is the household head’s ethnic background?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicspatient",
            name="pat_religion",
            field=models.CharField(
                choices=[
                    ("catholic", "Catholic"),
                    ("protestant", "Protestant"),
                    ("muslim", "Muslim"),
                    ("pentecostal", "Pentecostal"),
                    ("seventh_day_adventist", "Seventh day Adventist"),
                    ("no_religion", "No religion"),
                    ("OTHER", "Other, specify ..."),
                    ("muslim", "Muslim"),
                    ("seventh_day_adventist", "Seventh day Adventist"),
                    ("tz_assemblies_of_god", "Tanzanian Assemblies of God"),
                    ("catholic", "Catholic"),
                    ("lutheran", "Lutheran"),
                    ("moravian", "Moravian"),
                    ("pentecostal", "Pentecostal"),
                    ("african_inland_church", "African Inland Church"),
                    ("no_religion", "No religion"),
                    ("OTHER", "Other, specify ..."),
                ],
                max_length=25,
                verbose_name="How would you describe the household head’s religious orientation?",
            ),
        ),
    ]



# Generated by Django 4.2.10 on 2024-03-07 19:17

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("intecomm_lists", "0012_visitreasons_custom_name"),
        ("edc_he", "0007_remove_education_edc_he_educ_name_64c0c9_idx_and_more"),
        ("intecomm_subject", "0150_alter_careseekinga_money_sources_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="drugrefilldm",
            name="modifications_reason",
            field=models.ManyToManyField(
                blank=True,
                to="intecomm_lists.rxmodificationreasons",
                verbose_name="Why did the patient's previous prescription change?",
            ),
        ),
        migrations.AlterField(
            model_name="drugrefilldm",
            name="rx_modified",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                verbose_name="Was the patient's prescription changed at this visit compared with their prescription at the previous visit?",
            ),
        ),
        migrations.AlterField(
            model_name="drugrefillhiv",
            name="modifications_reason",
            field=models.ManyToManyField(
                blank=True,
                to="intecomm_lists.rxmodificationreasons",
                verbose_name="Why did the patient's previous prescription change?",
            ),
        ),
        migrations.AlterField(
            model_name="drugrefillhiv",
            name="rx_modified",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                verbose_name="Was the patient's prescription changed at this visit compared with their prescription at the previous visit?",
            ),
        ),
        migrations.AlterField(
            model_name="drugrefillhtn",
            name="modifications_reason",
            field=models.ManyToManyField(
                blank=True,
                to="intecomm_lists.rxmodificationreasons",
                verbose_name="Why did the patient's previous prescription change?",
            ),
        ),
        migrations.AlterField(
            model_name="drugrefillhtn",
            name="rx_modified",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                verbose_name="Was the patient's prescription changed at this visit compared with their prescription at the previous visit?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_employment_status",
            field=models.CharField(
                choices=[
                    ("1", "Full time employed"),
                    ("2", "Regular part time employed "),
                    ("9", "Self employed "),
                    ("3", "Irregular/ occasional/ day worker employment"),
                    ("4", "Non-paid/ voluntary role "),
                    ("5", "Student"),
                    ("6", "Homemaker"),
                    ("7", "Unemployed (able to work)"),
                    ("8", "Unemployed (unable to work)"),
                    ("dont_know", "Don't know"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="Not applicable if patient is head of household. If the household head has more than one job, think about their MAIN source of employment",
                max_length=25,
                verbose_name="Household head's employment status",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_employment_type",
            field=models.ForeignKey(
                help_text="Not applicable if patient is head of household",
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_he.employmenttype",
                verbose_name="Household head's type of employment",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_employment_type_old",
            field=models.CharField(
                default="QUESTION_RETIRED",
                editable=False,
                max_length=25,
                verbose_name="Household head's type of employment",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_ethnicity",
            field=models.ForeignKey(
                help_text="Not applicable if patient is head of household",
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_he.ethnicities",
                verbose_name="What is the household head's ethnic background?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_ethnicity_old",
            field=models.CharField(
                default="QUESTION_RETIRED",
                editable=False,
                max_length=25,
                verbose_name="What is the household head's ethnic background?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_insurance",
            field=models.ManyToManyField(
                help_text="Not applicable if patient is head of household",
                related_name="+",
                to="edc_he.insurancetypes",
                verbose_name="Household head's health insurance and 'club' status ",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_marital_status",
            field=models.CharField(
                choices=[
                    ("1", "Never Married (but not co-habiting)"),
                    ("2", "Co-habiting"),
                    ("3", "Currently Married"),
                    ("4", "Separated/Divorced"),
                    ("5", "Widowed"),
                    ("OTHER", "Other, specify ..."),
                    ("dont_know", "Don't know"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="Not applicable if patient is head of household",
                max_length=25,
                verbose_name="Household head's marital status",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_religion",
            field=models.ForeignKey(
                help_text="Not applicable if patient is head of household",
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="edc_he.religions",
                verbose_name="How would you describe the household head's religious orientation?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_religion_old",
            field=models.CharField(
                default="QUESTION_RETIRED",
                editable=False,
                max_length=25,
                verbose_name="How would you describe the household head's religious orientation?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="external_dependents",
            field=models.IntegerField(
                help_text="Insert '0' if no dependents other than the members in the household roster",
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(15),
                ],
                verbose_name="Outside of this household, how many other people depend on this household's income?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicsincome",
            name="financial_status_compare",
            field=models.CharField(
                choices=[
                    ("1", "Among most wealthy"),
                    ("2", "Above average "),
                    ("3", "Average wealth"),
                    ("4", "Below average"),
                    ("5", "Among least wealthy"),
                    ("DWTA", "Don't want to answer"),
                ],
                max_length=25,
                verbose_name="How would you rate your household's financial situation compared with others in your community?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldrugrefilldm",
            name="rx_modified",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                verbose_name="Was the patient's prescription changed at this visit compared with their prescription at the previous visit?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldrugrefillhiv",
            name="rx_modified",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                verbose_name="Was the patient's prescription changed at this visit compared with their prescription at the previous visit?",
            ),
        ),
        migrations.AlterField(
            model_name="historicaldrugrefillhtn",
            name="rx_modified",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                verbose_name="Was the patient's prescription changed at this visit compared with their prescription at the previous visit?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_employment_status",
            field=models.CharField(
                choices=[
                    ("1", "Full time employed"),
                    ("2", "Regular part time employed "),
                    ("9", "Self employed "),
                    ("3", "Irregular/ occasional/ day worker employment"),
                    ("4", "Non-paid/ voluntary role "),
                    ("5", "Student"),
                    ("6", "Homemaker"),
                    ("7", "Unemployed (able to work)"),
                    ("8", "Unemployed (unable to work)"),
                    ("dont_know", "Don't know"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="Not applicable if patient is head of household. If the household head has more than one job, think about their MAIN source of employment",
                max_length=25,
                verbose_name="Household head's employment status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_employment_type",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                help_text="Not applicable if patient is head of household",
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="edc_he.employmenttype",
                verbose_name="Household head's type of employment",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_employment_type_old",
            field=models.CharField(
                default="QUESTION_RETIRED",
                editable=False,
                max_length=25,
                verbose_name="Household head's type of employment",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_ethnicity",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                help_text="Not applicable if patient is head of household",
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="edc_he.ethnicities",
                verbose_name="What is the household head's ethnic background?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_ethnicity_old",
            field=models.CharField(
                default="QUESTION_RETIRED",
                editable=False,
                max_length=25,
                verbose_name="What is the household head's ethnic background?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_marital_status",
            field=models.CharField(
                choices=[
                    ("1", "Never Married (but not co-habiting)"),
                    ("2", "Co-habiting"),
                    ("3", "Currently Married"),
                    ("4", "Separated/Divorced"),
                    ("5", "Widowed"),
                    ("OTHER", "Other, specify ..."),
                    ("dont_know", "Don't know"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                help_text="Not applicable if patient is head of household",
                max_length=25,
                verbose_name="Household head's marital status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_religion",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                help_text="Not applicable if patient is head of household",
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="edc_he.religions",
                verbose_name="How would you describe the household head's religious orientation?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_religion_old",
            field=models.CharField(
                default="QUESTION_RETIRED",
                editable=False,
                max_length=25,
                verbose_name="How would you describe the household head's religious orientation?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsincome",
            name="external_dependents",
            field=models.IntegerField(
                help_text="Insert '0' if no dependents other than the members in the household roster",
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(15),
                ],
                verbose_name="Outside of this household, how many other people depend on this household's income?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicsincome",
            name="financial_status_compare",
            field=models.CharField(
                choices=[
                    ("1", "Among most wealthy"),
                    ("2", "Above average "),
                    ("3", "Average wealth"),
                    ("4", "Below average"),
                    ("5", "Among least wealthy"),
                    ("DWTA", "Don't want to answer"),
                ],
                max_length=25,
                verbose_name="How would you rate your household's financial situation compared with others in your community?",
            ),
        ),
    ]



from .community_care_location_admin import CommunityCareLocationAdmin
from .patient_followup_call_admin import PatientFollowupCallAdmin
from .patient_group_admin import PatientGroupAdmin
from .patient_group_appointment_admin import PatientGroupAppointmentAdmin
from .patient_group_meeting_admin import PatientGroupMeetingAdmin



# Generated by Django 4.2.3 on 2023-08-03 03:33

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0077_healtheconomicshouseholdhead_hoh_religion_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="healtheconomicshouseholdhead",
            name="hoh_religion_old",
            field=models.CharField(
                max_length=25,
                verbose_name="How would you describe the household head’s religious orientation?",
            ),
        ),
        migrations.AlterField(
            model_name="healtheconomicspatient",
            name="pat_religion_old",
            field=models.CharField(
                max_length=25,
                verbose_name="How would you describe your religious orientation?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicshouseholdhead",
            name="hoh_religion_old",
            field=models.CharField(
                max_length=25,
                verbose_name="How would you describe the household head’s religious orientation?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalhealtheconomicspatient",
            name="pat_religion_old",
            field=models.CharField(
                max_length=25,
                verbose_name="How would you describe your religious orientation?",
            ),
        ),
    ]



# Generated by Django 4.2.7 on 2023-11-14 18:51

from django.db import migrations
import edc_visit_tracking.managers
import intecomm_subject.models.subject_visit


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0122_remove_historicalnextappointment_info_source_old_and_more"),
    ]

    operations = [
        migrations.AlterModelManagers(
            name="subjectvisit",
            managers=[
                ("objects", edc_visit_tracking.managers.VisitModelManager()),
                ("on_site", intecomm_subject.models.subject_visit.CurrentSiteManager()),
            ],
        ),
    ]



# Generated by Django 4.2.9 on 2024-02-06 04:26

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0141_remove_viralloadresult_requisition_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalhtninitialreview",
            name="managed_by_old",
            field=models.CharField(
                blank=True,
                choices=[
                    ("drugs", "Drugs / Medicine"),
                    ("diet_lifestyle", "Diet and lifestyle alone"),
                ],
                default="N/A",
                help_text="If patient is taking medication, complete the Drug Refill CRF.",
                max_length=15,
                verbose_name="How is the patient's hypertension managed?",
            ),
        ),
        migrations.AlterField(
            model_name="htninitialreview",
            name="managed_by_old",
            field=models.CharField(
                blank=True,
                choices=[
                    ("drugs", "Drugs / Medicine"),
                    ("diet_lifestyle", "Diet and lifestyle alone"),
                ],
                default="N/A",
                help_text="If patient is taking medication, complete the Drug Refill CRF.",
                max_length=15,
                verbose_name="How is the patient's hypertension managed?",
            ),
        ),
    ]



# Generated by Django 4.2.9 on 2024-02-01 14:14

import django.core.validators
from django.db import migrations, models
import edc_model.validators.date


class Migration(migrations.Migration):
    dependencies = [
        ("intecomm_subject", "0139_alter_bloodresultsins_fasting_duration_minutes_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalhivreview",
            name="drawn_date",
            field=models.DateField(
                blank=True,
                help_text="If exact date not known, please estimate",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date specimen drawn",
            ),
        ),
        migrations.AddField(
            model_name="historicalhivreview",
            name="has_vl",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("PENDING", "Pending"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="Is the patient's most recent viral load result available?",
            ),
        ),
        migrations.AddField(
            model_name="historicalhivreview",
            name="vl",
            field=models.IntegerField(
                blank=True,
                help_text="copies/mL",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(9999999),
                ],
                verbose_name="Most recent viral load",
            ),
        ),
        migrations.AddField(
            model_name="historicalhivreview",
            name="vl_quantifier",
            field=models.CharField(
                choices=[("=", "="), (">", ">"), ("<", "<"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=10,
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="hivreview",
            name="drawn_date",
            field=models.DateField(
                blank=True,
                help_text="If exact date not known, please estimate",
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date specimen drawn",
            ),
        ),
        migrations.AddField(
            model_name="hivreview",
            name="has_vl",
            field=models.CharField(
                choices=[
                    ("Yes", "Yes"),
                    ("No", "No"),
                    ("PENDING", "Pending"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="Is the patient's most recent viral load result available?",
            ),
        ),
        migrations.AddField(
            model_name="hivreview",
            name="vl",
            field=models.IntegerField(
                blank=True,
                help_text="copies/mL",
                null=True,
                validators=[
                    django.core.validators.MinValueValidator(0),
                    django.core.validators.MaxValueValidator(9999999),
                ],
                verbose_name="Most recent viral load",
            ),
        ),
        migrations.AddField(
            model_name="hivreview",
            name="vl_quantifier",
            field=models.CharField(
                choices=[("=", "="), (">", ">"), ("<", "<"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=10,
                null=True,
            ),
        ),
    ]
